# -*- coding: utf-8 -*-

# File generated according to SMachineDimension.ui
# WARNING! All changes made in this file will be lost!
## WARNING! All changes made in this file will be lost when recompiling UI file!
################################################################################

from PySide2.QtCore import *
from PySide2.QtGui import *
from PySide2.QtWidgets import *

from .....GUI.Tools.FloatEdit import FloatEdit
from .....GUI.Dialog.DMatLib.WMatSelect.WMatSelectV import WMatSelectV

from pyleecan.GUI.Resources import pyleecan_rc


class Ui_SMachineDimension(object):
    def setupUi(self, SMachineDimension):
        if not SMachineDimension.objectName():
            SMachineDimension.setObjectName(u"SMachineDimension")
        SMachineDimension.resize(919, 550)
        SMachineDimension.setMinimumSize(QSize(650, 550))
        self.horizontalLayout_2 = QHBoxLayout(SMachineDimension)
        self.horizontalLayout_2.setObjectName(u"horizontalLayout_2")
        self.img_machine = QLabel(SMachineDimension)
        self.img_machine.setObjectName(u"img_machine")
        self.img_machine.setMinimumSize(QSize(0, 0))
        self.img_machine.setMaximumSize(QSize(16777215, 16777215))
        self.img_machine.setPixmap(
            QPixmap(
                u":/images/images/MachineSetup/MachineDimension/Dimension_Shaft_Rotor_Stator.png"
            )
        )
        self.img_machine.setScaledContents(True)

        self.horizontalLayout_2.addWidget(self.img_machine)

        self.widget = QWidget(SMachineDimension)
        self.widget.setObjectName(u"widget")
        self.widget.setMinimumSize(QSize(250, 0))
        self.widget.setMaximumSize(QSize(250, 16777215))
        self.verticalLayout_3 = QVBoxLayout(self.widget)
        self.verticalLayout_3.setObjectName(u"verticalLayout_3")
        self.g_stator = QGroupBox(self.widget)
        self.g_stator.setObjectName(u"g_stator")
        self.g_stator.setMinimumSize(QSize(150, 0))
        self.g_stator.setMaximumSize(QSize(16777215, 16777215))
        self.gridLayout_2 = QGridLayout(self.g_stator)
        self.gridLayout_2.setObjectName(u"gridLayout_2")
        self.in_SRext = QLabel(self.g_stator)
        self.in_SRext.setObjectName(u"in_SRext")
        self.in_SRext.setMinimumSize(QSize(30, 0))

        self.gridLayout_2.addWidget(self.in_SRext, 0, 0, 1, 1)

        self.lf_SRext = FloatEdit(self.g_stator)
        self.lf_SRext.setObjectName(u"lf_SRext")
        self.lf_SRext.setMaximumSize(QSize(100, 16777215))

        self.gridLayout_2.addWidget(self.lf_SRext, 0, 1, 1, 1)

        self.unit_SRext = QLabel(self.g_stator)
        self.unit_SRext.setObjectName(u"unit_SRext")
        sizePolicy = QSizePolicy(QSizePolicy.Fixed, QSizePolicy.Preferred)
        sizePolicy.setHorizontalStretch(0)
        sizePolicy.setVerticalStretch(0)
        sizePolicy.setHeightForWidth(self.unit_SRext.sizePolicy().hasHeightForWidth())
        self.unit_SRext.setSizePolicy(sizePolicy)

        self.gridLayout_2.addWidget(self.unit_SRext, 0, 2, 1, 1)

        self.in_SRint = QLabel(self.g_stator)
        self.in_SRint.setObjectName(u"in_SRint")
        self.in_SRint.setMinimumSize(QSize(30, 0))

        self.gridLayout_2.addWidget(self.in_SRint, 1, 0, 1, 1)

        self.lf_SRint = FloatEdit(self.g_stator)
        self.lf_SRint.setObjectName(u"lf_SRint")
        self.lf_SRint.setMaximumSize(QSize(100, 100))

        self.gridLayout_2.addWidget(self.lf_SRint, 1, 1, 1, 1)

        self.unit_SRint = QLabel(self.g_stator)
        self.unit_SRint.setObjectName(u"unit_SRint")
        sizePolicy.setHeightForWidth(self.unit_SRint.sizePolicy().hasHeightForWidth())
        self.unit_SRint.setSizePolicy(sizePolicy)

        self.gridLayout_2.addWidget(self.unit_SRint, 1, 2, 1, 1)

        self.verticalLayout_3.addWidget(self.g_stator)

        self.g_rotor = QGroupBox(self.widget)
        self.g_rotor.setObjectName(u"g_rotor")
        self.gridLayout_3 = QGridLayout(self.g_rotor)
        self.gridLayout_3.setObjectName(u"gridLayout_3")
        self.in_RRext = QLabel(self.g_rotor)
        self.in_RRext.setObjectName(u"in_RRext")
        self.in_RRext.setMinimumSize(QSize(30, 0))

        self.gridLayout_3.addWidget(self.in_RRext, 0, 0, 1, 1)

        self.lf_RRext = FloatEdit(self.g_rotor)
        self.lf_RRext.setObjectName(u"lf_RRext")
        self.lf_RRext.setMaximumSize(QSize(100, 16777215))

        self.gridLayout_3.addWidget(self.lf_RRext, 0, 1, 1, 1)

        self.unit_RRext = QLabel(self.g_rotor)
        self.unit_RRext.setObjectName(u"unit_RRext")
        sizePolicy.setHeightForWidth(self.unit_RRext.sizePolicy().hasHeightForWidth())
        self.unit_RRext.setSizePolicy(sizePolicy)

        self.gridLayout_3.addWidget(self.unit_RRext, 0, 2, 1, 1)

        self.in_RRint = QLabel(self.g_rotor)
        self.in_RRint.setObjectName(u"in_RRint")
        self.in_RRint.setMinimumSize(QSize(30, 0))

        self.gridLayout_3.addWidget(self.in_RRint, 1, 0, 1, 1)

        self.lf_RRint = FloatEdit(self.g_rotor)
        self.lf_RRint.setObjectName(u"lf_RRint")
        self.lf_RRint.setMaximumSize(QSize(100, 100))

        self.gridLayout_3.addWidget(self.lf_RRint, 1, 1, 1, 1)

        self.unit_RRint = QLabel(self.g_rotor)
        self.unit_RRint.setObjectName(u"unit_RRint")
        sizePolicy.setHeightForWidth(self.unit_RRint.sizePolicy().hasHeightForWidth())
        self.unit_RRint.setSizePolicy(sizePolicy)

        self.gridLayout_3.addWidget(self.unit_RRint, 1, 2, 1, 1)

        self.verticalLayout_3.addWidget(self.g_rotor)

        self.out_airgap = QLabel(self.widget)
        self.out_airgap.setObjectName(u"out_airgap")

        self.verticalLayout_3.addWidget(self.out_airgap)

        self.g_shaft = QGroupBox(self.widget)
        self.g_shaft.setObjectName(u"g_shaft")
        self.g_shaft.setMinimumSize(QSize(0, 0))
        self.g_shaft.setCheckable(True)
        self.verticalLayout = QVBoxLayout(self.g_shaft)
        self.verticalLayout.setObjectName(u"verticalLayout")
        self.w_mat_0 = WMatSelectV(self.g_shaft)
        self.w_mat_0.setObjectName(u"w_mat_0")
        self.w_mat_0.setMinimumSize(QSize(100, 0))

        self.verticalLayout.addWidget(self.w_mat_0)

        self.out_Drsh = QLabel(self.g_shaft)
        self.out_Drsh.setObjectName(u"out_Drsh")

        self.verticalLayout.addWidget(self.out_Drsh)

        self.verticalLayout_3.addWidget(self.g_shaft)

        self.g_frame = QGroupBox(self.widget)
        self.g_frame.setObjectName(u"g_frame")
        self.g_frame.setCheckable(True)
        self.verticalLayout_2 = QVBoxLayout(self.g_frame)
        self.verticalLayout_2.setObjectName(u"verticalLayout_2")
        self.w_mat_1 = WMatSelectV(self.g_frame)
        self.w_mat_1.setObjectName(u"w_mat_1")
        self.w_mat_1.setMinimumSize(QSize(100, 0))

        self.verticalLayout_2.addWidget(self.w_mat_1)

        self.gridLayout = QGridLayout()
        self.gridLayout.setObjectName(u"gridLayout")
        self.lf_Lfra = FloatEdit(self.g_frame)
        self.lf_Lfra.setObjectName(u"lf_Lfra")
        self.lf_Lfra.setMaximumSize(QSize(100, 16777215))

        self.gridLayout.addWidget(self.lf_Lfra, 1, 1, 1, 1)

        self.in_Lfra = QLabel(self.g_frame)
        self.in_Lfra.setObjectName(u"in_Lfra")

        self.gridLayout.addWidget(self.in_Lfra, 1, 0, 1, 1)

        self.lf_Wfra = FloatEdit(self.g_frame)
        self.lf_Wfra.setObjectName(u"lf_Wfra")
        self.lf_Wfra.setMaximumSize(QSize(100, 16777215))

        self.gridLayout.addWidget(self.lf_Wfra, 0, 1, 1, 1)

        self.in_Wfra = QLabel(self.g_frame)
        self.in_Wfra.setObjectName(u"in_Wfra")

        self.gridLayout.addWidget(self.in_Wfra, 0, 0, 1, 1)

        self.unit_Lfra = QLabel(self.g_frame)
        self.unit_Lfra.setObjectName(u"unit_Lfra")

        self.gridLayout.addWidget(self.unit_Lfra, 1, 2, 1, 1)

        self.unit_Wfra = QLabel(self.g_frame)
        self.unit_Wfra.setObjectName(u"unit_Wfra")

        self.gridLayout.addWidget(self.unit_Wfra, 0, 2, 1, 1)

        self.verticalLayout_2.addLayout(self.gridLayout)

        self.verticalLayout_3.addWidget(self.g_frame)

        self.verticalSpacer = QSpacerItem(
            20, 40, QSizePolicy.Minimum, QSizePolicy.Expanding
        )

        self.verticalLayout_3.addItem(self.verticalSpacer)

        self.horizontalLayout = QHBoxLayout()
        self.horizontalLayout.setObjectName(u"horizontalLayout")
        self.horizontalSpacer = QSpacerItem(
            40, 20, QSizePolicy.Expanding, QSizePolicy.Minimum
        )

        self.horizontalLayout.addItem(self.horizontalSpacer)

        self.b_previous = QPushButton(self.widget)
        self.b_previous.setObjectName(u"b_previous")

        self.horizontalLayout.addWidget(self.b_previous)

        self.b_next = QPushButton(self.widget)
        self.b_next.setObjectName(u"b_next")

        self.horizontalLayout.addWidget(self.b_next)

        self.verticalLayout_3.addLayout(self.horizontalLayout)

        self.horizontalLayout_2.addWidget(self.widget)

        self.retranslateUi(SMachineDimension)
        self.g_shaft.toggled.connect(self.out_Drsh.setVisible)
        self.g_frame.toggled.connect(self.in_Lfra.setVisible)
        self.g_frame.toggled.connect(self.unit_Lfra.setVisible)

        QMetaObject.connectSlotsByName(SMachineDimension)

    # setupUi

    def retranslateUi(self, SMachineDimension):
        SMachineDimension.setWindowTitle(
            QCoreApplication.translate("SMachineDimension", u"Form", None)
        )
        self.img_machine.setText("")
        self.g_stator.setTitle(
            QCoreApplication.translate("SMachineDimension", u"Stator", None)
        )
        # if QT_CONFIG(tooltip)
        self.in_SRext.setToolTip(
            QCoreApplication.translate(
                "SMachineDimension", u"Stator external radius", None
            )
        )
        # endif // QT_CONFIG(tooltip)
        self.in_SRext.setText(
            QCoreApplication.translate("SMachineDimension", u"Rext", None)
        )
        # if QT_CONFIG(tooltip)
        self.lf_SRext.setToolTip(
            QCoreApplication.translate(
                "SMachineDimension", u"Stator external radius", None
            )
        )
        # endif // QT_CONFIG(tooltip)
        self.unit_SRext.setText(
            QCoreApplication.translate("SMachineDimension", u"m", None)
        )
        # if QT_CONFIG(tooltip)
        self.in_SRint.setToolTip(
            QCoreApplication.translate(
                "SMachineDimension", u"Stator internal radius", None
            )
        )
        # endif // QT_CONFIG(tooltip)
        self.in_SRint.setText(
            QCoreApplication.translate("SMachineDimension", u"Rint", None)
        )
        # if QT_CONFIG(tooltip)
        self.lf_SRint.setToolTip(
            QCoreApplication.translate(
                "SMachineDimension", u"Stator internal radius", None
            )
        )
        # endif // QT_CONFIG(tooltip)
        self.unit_SRint.setText(
            QCoreApplication.translate("SMachineDimension", u"m", None)
        )
        self.g_rotor.setTitle(
            QCoreApplication.translate("SMachineDimension", u"Rotor", None)
        )
        # if QT_CONFIG(tooltip)
        self.in_RRext.setToolTip(
            QCoreApplication.translate(
                "SMachineDimension", u"Rotor external radius", None
            )
        )
        # endif // QT_CONFIG(tooltip)
        self.in_RRext.setText(
            QCoreApplication.translate("SMachineDimension", u"Rext", None)
        )
        # if QT_CONFIG(tooltip)
        self.lf_RRext.setToolTip(
            QCoreApplication.translate(
                "SMachineDimension", u"Rotor external radius", None
            )
        )
        # endif // QT_CONFIG(tooltip)
        self.unit_RRext.setText(
            QCoreApplication.translate("SMachineDimension", u"m", None)
        )
        # if QT_CONFIG(tooltip)
        self.in_RRint.setToolTip(
            QCoreApplication.translate(
                "SMachineDimension", u"Rotor internal radius", None
            )
        )
        # endif // QT_CONFIG(tooltip)
        self.in_RRint.setText(
            QCoreApplication.translate("SMachineDimension", u"Rint", None)
        )
        # if QT_CONFIG(tooltip)
        self.lf_RRint.setToolTip(
            QCoreApplication.translate(
                "SMachineDimension", u"Rotor internal radius", None
            )
        )
        # endif // QT_CONFIG(tooltip)
        self.unit_RRint.setText(
            QCoreApplication.translate("SMachineDimension", u"m", None)
        )
        # if QT_CONFIG(tooltip)
        self.out_airgap.setToolTip(
            QCoreApplication.translate(
                "SMachineDimension",
                u"mechanical airgap width (distance between stator bore and rotor bore radii)",
                None,
            )
        )
        # endif // QT_CONFIG(tooltip)
        self.out_airgap.setText(
            QCoreApplication.translate("SMachineDimension", u"airgap = ", None)
        )
        self.g_shaft.setTitle(
            QCoreApplication.translate("SMachineDimension", u"Shaft", None)
        )
        # if QT_CONFIG(tooltip)
        self.out_Drsh.setToolTip(
            QCoreApplication.translate("SMachineDimension", u"Shaft Diameter", None)
        )
        # endif // QT_CONFIG(tooltip)
        self.out_Drsh.setText(
            QCoreApplication.translate(
                "SMachineDimension", u"Drsh = 2*Rotor.Rint", None
            )
        )
        self.g_frame.setTitle(
            QCoreApplication.translate("SMachineDimension", u"Frame", None)
        )
        # if QT_CONFIG(tooltip)
        self.lf_Lfra.setToolTip(
            QCoreApplication.translate("SMachineDimension", u"Frame length", None)
        )
        # endif // QT_CONFIG(tooltip)
        # if QT_CONFIG(whatsthis)
        self.lf_Lfra.setWhatsThis(
            QCoreApplication.translate("SMachineDimension", u"Frame length", None)
        )
        # endif // QT_CONFIG(whatsthis)
        # if QT_CONFIG(tooltip)
        self.in_Lfra.setToolTip(
            QCoreApplication.translate("SMachineDimension", u"Frame length", None)
        )
        # endif // QT_CONFIG(tooltip)
        # if QT_CONFIG(whatsthis)
        self.in_Lfra.setWhatsThis(
            QCoreApplication.translate("SMachineDimension", u"Frame length", None)
        )
        # endif // QT_CONFIG(whatsthis)
        self.in_Lfra.setText(
            QCoreApplication.translate("SMachineDimension", u"Lfra", None)
        )
        # if QT_CONFIG(tooltip)
        self.lf_Wfra.setToolTip(
            QCoreApplication.translate("SMachineDimension", u"Frame width", None)
        )
        # endif // QT_CONFIG(tooltip)
        # if QT_CONFIG(whatsthis)
        self.lf_Wfra.setWhatsThis(
            QCoreApplication.translate("SMachineDimension", u"Frame width", None)
        )
        # endif // QT_CONFIG(whatsthis)
        # if QT_CONFIG(tooltip)
        self.in_Wfra.setToolTip(
            QCoreApplication.translate("SMachineDimension", u"Frame width", None)
        )
        # endif // QT_CONFIG(tooltip)
        # if QT_CONFIG(whatsthis)
        self.in_Wfra.setWhatsThis(
            QCoreApplication.translate("SMachineDimension", u"Frame width", None)
        )
        # endif // QT_CONFIG(whatsthis)
        self.in_Wfra.setText(
            QCoreApplication.translate("SMachineDimension", u"Wfra", None)
        )
        # if QT_CONFIG(tooltip)
        self.unit_Lfra.setToolTip(
            QCoreApplication.translate("SMachineDimension", u"Frame length", None)
        )
        # endif // QT_CONFIG(tooltip)
        # if QT_CONFIG(whatsthis)
        self.unit_Lfra.setWhatsThis(
            QCoreApplication.translate("SMachineDimension", u"Frame length", None)
        )
        # endif // QT_CONFIG(whatsthis)
        self.unit_Lfra.setText(
            QCoreApplication.translate("SMachineDimension", u"m", None)
        )
        # if QT_CONFIG(tooltip)
        self.unit_Wfra.setToolTip(
            QCoreApplication.translate("SMachineDimension", u"Frame width", None)
        )
        # endif // QT_CONFIG(tooltip)
        # if QT_CONFIG(whatsthis)
        self.unit_Wfra.setWhatsThis(
            QCoreApplication.translate("SMachineDimension", u"Frame width", None)
        )
        # endif // QT_CONFIG(whatsthis)
        self.unit_Wfra.setText(
            QCoreApplication.translate("SMachineDimension", u"m", None)
        )
        self.b_previous.setText(
            QCoreApplication.translate("SMachineDimension", u"Previous", None)
        )
        self.b_next.setText(
            QCoreApplication.translate("SMachineDimension", u"Next", None)
        )

    # retranslateUi



from ....Methods.Geometry.PolarArc import *


def check(self):
    """check if the PolarArc object is correct

    Parameters
    ----------
    self : PolarArc
        a PolarArc Object

    Returns
    -------
    None

    Raises
    ------
    AnglePolarArcError
        The angle of a polar arc must be different to 0

    """
    if self.angle == 0:
        raise AnglePolarArcError("The angle of a polar arc must be different to 0")



# -*- coding: utf-8 -*-
from math import gcd


def comp_periodicity(self):
    """Compute the (anti)-periodicities of the machine in time and space domain

    Parameters
    ----------
    self : Machine
        A Machine object

    Returns
    -------
    per : int
        Number of periodicities of the machine
    is_antisym : bool
        True if an anti-periodicity is possible after the periodicities
    """
    # Get stator (anti)-periodicity in spatial domain only (because stator is not moving regarding time)
    pera_s, is_antipera_s, _, _ = self.stator.comp_periodicity()

    # Get rotor (anti)-periodicities both in time and spatial domains
    pera_r, is_antipera_r, pert_r, is_antipert_r = self.rotor.comp_periodicity()

    return (gcd(pera_s, pera_r), is_antipera_s and is_antipera_r, pert_r, is_antipert_r)



from inspect import getfullargspec


def run(self, output):
    """Execute the plot contained in the PostPlot object and save it to save_path

    Parameters
    ----------
    self : PostPlot
        A PostPlot object
    output : Output
        an Output object

    """

    list_names = self.method.split(".")

    if len(list_names) == 1:
        # Getting the name of the plot method of the output or the object given by attribute
        plot_method = getattr(output, self.method)
    else:
        # Find object which contains the plot method if attribute is not None
        obj = output

        # Get successive objects to reach the one containing the method
        for i in range(len(list_names) - 1):
            obj = getattr(obj, list_names[i])

        # Getting the name of the plot method of the output or the object given by attribute
        plot_method = getattr(obj, list_names[-1])

    # Path to save the figure if save_path is not already in param_dict and is an argument of the plot method
    if (
        "save_path" not in self.param_dict
        and "save_path" in getfullargspec(plot_method)[5]
    ):
        # Get path of results folder in the Output
        result_path = output.get_path_result()

        # Check in case format begins with a "."
        if self.save_format[0] == ".":
            str_format = self.save_format
        else:
            str_format = "." + self.save_format

        # Build save_path of the picture
        self.param_dict["save_path"] = result_path + "/" + self.name + str_format

    # Execute plot method
    plot_method(
        *self.param_list,
        **self.param_dict,
    )



# -*- coding: utf-8 -*-


def comp_surface(self):
    """Compute the surface of all the axial ventilation ducts

    Parameters
    ----------
    self : VentilationTrap
        A VentilationTrap object

    Returns
    -------
    surface: float
        Axial ventilation ducts total surface [m**2]

    """

    self.check()

    return (self.W1 + self.W2) * 0.5 * self.D0 * self.Zh



from ....Methods import ParentMissingError


def get_Rbo(self):
    """Return the parent lamination bore radius

    Parameters
    ----------
    self : Hole
        A Hole object

    Returns
    -------
    Rbo: float
        The parent lamination bore radius [m]

    """

    if self.parent is not None:
        return self.parent.get_Rbo()
    else:
        raise ParentMissingError("Error: The hole is not inside a Lamination")



# -*- coding: utf-8 -*-


def get_machine_type(self):
    """Return a string with the may information about the machine architecture

    Parameters
    ----------
    self : MachineDFIM
        A MachineDFIM object

    Returns
    -------
    type_str: str
        DFIM Zs/Zr/p (int/ext rotor)

    """

    type_str = "DFIM "

    if self.stator.slot.Zs is not None:
        type_str += str(self.stator.slot.Zs) + "s / "
    else:
        type_str += "?s / "

    if self.rotor.slot.Zs is not None:
        type_str += str(self.rotor.slot.Zs) + "r / "
    else:
        type_str += "?r / "

    if self.stator.winding.p is not None:
        type_str += str(self.stator.winding.p) + "p"
    else:
        type_str += "?p"

    if self.stator.is_internal:
        type_str += " (ext rotor)"
    else:
        type_str += " (int rotor)"

    return type_str



# -*- coding: utf-8 -*-

import numpy as np


def add_point(self, coord):
    """Define a new PointMat object based on a set of elements.

    Parameters
    ----------
    self : PointMat
        an PointMat object
    element : Element
        an Element object

    Returns
    -------
    node: Node
        a Node object corresponding to Element

    """
    if self.is_exist(coord):
        return None
    else:
        if self.coordinate is not None and self.coordinate.size > 0:
            self.coordinate = np.vstack((self.coordinate, coord))
            new_ind = max(self.indice) + 1
            self.indice = np.concatenate((self.indice, np.array([new_ind], dtype=int)))
            self.nb_pt = self.nb_pt + 1
        else:
            self.coordinate = np.concatenate((self.coordinate, coord))
            new_ind = 0
            self.indice = np.array([], dtype=int)
            self.indice = np.concatenate((self.indice, np.array([new_ind], dtype=int)))
            self.nb_pt = self.nb_pt + 1

    return new_ind



from ....Methods.Geometry.Trapeze import *


def check(self):
    """Check if the Trapeze object is correct

    Parameters
    ----------
    self : Trapeze
        a Trapeze Object


    Returns
    -------
    None

    Raises
    ------
    TrapezeError
        the W1 base of Trapeze must be greater than 0
    TrapezeError
        the W2 base of Trapeze must be greater than 0

    """
    if self.W1 <= 0:
        raise TrapezeError("the W1 base of Trapeze must be greater than 0")

    if self.W2 <= 0:
        raise TrapezeError("the W2 base of Trapeze must be greater than 0")



def get_magnet_dict(self):
    """Return a dictionnary with all the magnets of the Hole

    Parameters
    ----------
    self : Hole
        A Hole object

    Returns
    -------
    magnet_dict : {Magnet}
        Dictionnary of magnet (key = magnet_X, value= Magnet or None)
    """

    if hasattr(self, "magnet_dict"):
        return self.magnet_dict
    else:
        ii = 0
        magnet_dict = dict()
        while hasattr(self, "magnet_" + str(ii)):
            magnet_dict["magnet_" + str(ii)] = getattr(self, "magnet_" + str(ii))
            ii += 1
        return magnet_dict



# -*- coding: utf-8 -*-
# File generated according to Generator/ClassesRef/Output/XOutput.csv
# WARNING! All changes made in this file will be lost!
"""Method code available at https://github.com/Eomys/pyleecan/tree/master/pyleecan/Methods/Output/XOutput
"""

from os import linesep
from sys import getsizeof
from logging import getLogger
from ._check import check_var, raise_
from ..Functions.get_logger import get_logger
from ..Functions.save import save
from ..Functions.copy import copy
from ..Functions.load import load_init_dict
from ..Functions.Load.import_class import import_class
from .Output import Output

# Import all class method
# Try/catch to remove unnecessary dependencies in unused method
try:
    from ..Methods.Output.XOutput.__delitem__ import __delitem__
except ImportError as error:
    __delitem__ = error

try:
    from ..Methods.Output.XOutput.__getitem__ import __getitem__
except ImportError as error:
    __getitem__ = error

try:
    from ..Methods.Output.XOutput.__iter__ import __iter__
except ImportError as error:
    __iter__ = error

try:
    from ..Methods.Output.XOutput.__len__ import __len__
except ImportError as error:
    __len__ = error

try:
    from ..Methods.Output.XOutput.__missing__ import __missing__
except ImportError as error:
    __missing__ = error

try:
    from ..Methods.Output.XOutput.__next__ import __next__
except ImportError as error:
    __next__ = error

try:
    from ..Methods.Output.XOutput.__reversed__ import __reversed__
except ImportError as error:
    __reversed__ = error

try:
    from ..Methods.Output.XOutput.__setitem__ import __setitem__
except ImportError as error:
    __setitem__ = error

try:
    from ..Methods.Output.XOutput.count import count
except ImportError as error:
    count = error

try:
    from ..Methods.Output.XOutput.insert import insert
except ImportError as error:
    insert = error

try:
    from ..Methods.Output.XOutput.pop import pop
except ImportError as error:
    pop = error

try:
    from ..Methods.Output.XOutput.remove import remove
except ImportError as error:
    remove = error

try:
    from ..Methods.Output.XOutput.append import append
except ImportError as error:
    append = error

try:
    from ..Methods.Output.XOutput.get_simu import get_simu
except ImportError as error:
    get_simu = error

try:
    from ..Methods.Output.XOutput.keys import keys
except ImportError as error:
    keys = error

try:
    from ..Methods.Output.XOutput.items import items
except ImportError as error:
    items = error

try:
    from ..Methods.Output.XOutput.get_param_simu import get_param_simu
except ImportError as error:
    get_param_simu = error

try:
    from ..Methods.Output.XOutput.get_paramexplorer import get_paramexplorer
except ImportError as error:
    get_paramexplorer = error

try:
    from ..Methods.Output.XOutput.plot_multi import plot_multi
except ImportError as error:
    plot_multi = error

try:
    from ..Methods.Output.XOutput.plot_pareto import plot_pareto
except ImportError as error:
    plot_pareto = error

try:
    from ..Methods.Output.XOutput.plot_generation import plot_generation
except ImportError as error:
    plot_generation = error

try:
    from ..Methods.Output.XOutput.get_pareto_index import get_pareto_index
except ImportError as error:
    get_pareto_index = error

try:
    from ..Methods.Output.XOutput.print_memory import print_memory
except ImportError as error:
    print_memory = error

try:
    from ..Methods.Output.XOutput.get_xoutput_ref import get_xoutput_ref
except ImportError as error:
    get_xoutput_ref = error


from ._check import InitUnKnowClassError
from .ParamExplorer import ParamExplorer
from .Output import Output
from .DataKeeper import DataKeeper
from .Simulation import Simulation
from .OutGeo import OutGeo
from .OutElec import OutElec
from .OutMag import OutMag
from .OutStruct import OutStruct
from .OutPost import OutPost
from .OutForce import OutForce
from .OutLoss import OutLoss


class XOutput(Output):
    """XOutput object: gather all the outputs of all the modules for multiple simulations"""

    VERSION = 1

    # Check ImportError to remove unnecessary dependencies in unused method
    # cf Methods.Output.XOutput.__delitem__
    if isinstance(__delitem__, ImportError):
        __delitem__ = property(
            fget=lambda x: raise_(
                ImportError("Can't use XOutput method __delitem__: " + str(__delitem__))
            )
        )
    else:
        __delitem__ = __delitem__
    # cf Methods.Output.XOutput.__getitem__
    if isinstance(__getitem__, ImportError):
        __getitem__ = property(
            fget=lambda x: raise_(
                ImportError("Can't use XOutput method __getitem__: " + str(__getitem__))
            )
        )
    else:
        __getitem__ = __getitem__
    # cf Methods.Output.XOutput.__iter__
    if isinstance(__iter__, ImportError):
        __iter__ = property(
            fget=lambda x: raise_(
                ImportError("Can't use XOutput method __iter__: " + str(__iter__))
            )
        )
    else:
        __iter__ = __iter__
    # cf Methods.Output.XOutput.__len__
    if isinstance(__len__, ImportError):
        __len__ = property(
            fget=lambda x: raise_(
                ImportError("Can't use XOutput method __len__: " + str(__len__))
            )
        )
    else:
        __len__ = __len__
    # cf Methods.Output.XOutput.__missing__
    if isinstance(__missing__, ImportError):
        __missing__ = property(
            fget=lambda x: raise_(
                ImportError("Can't use XOutput method __missing__: " + str(__missing__))
            )
        )
    else:
        __missing__ = __missing__
    # cf Methods.Output.XOutput.__next__
    if isinstance(__next__, ImportError):
        __next__ = property(
            fget=lambda x: raise_(
                ImportError("Can't use XOutput method __next__: " + str(__next__))
            )
        )
    else:
        __next__ = __next__
    # cf Methods.Output.XOutput.__reversed__
    if isinstance(__reversed__, ImportError):
        __reversed__ = property(
            fget=lambda x: raise_(
                ImportError(
                    "Can't use XOutput method __reversed__: " + str(__reversed__)
                )
            )
        )
    else:
        __reversed__ = __reversed__
    # cf Methods.Output.XOutput.__setitem__
    if isinstance(__setitem__, ImportError):
        __setitem__ = property(
            fget=lambda x: raise_(
                ImportError("Can't use XOutput method __setitem__: " + str(__setitem__))
            )
        )
    else:
        __setitem__ = __setitem__
    # cf Methods.Output.XOutput.count
    if isinstance(count, ImportError):
        count = property(
            fget=lambda x: raise_(
                ImportError("Can't use XOutput method count: " + str(count))
            )
        )
    else:
        count = count
    # cf Methods.Output.XOutput.insert
    if isinstance(insert, ImportError):
        insert = property(
            fget=lambda x: raise_(
                ImportError("Can't use XOutput method insert: " + str(insert))
            )
        )
    else:
        insert = insert
    # cf Methods.Output.XOutput.pop
    if isinstance(pop, ImportError):
        pop = property(
            fget=lambda x: raise_(
                ImportError("Can't use XOutput method pop: " + str(pop))
            )
        )
    else:
        pop = pop
    # cf Methods.Output.XOutput.remove
    if isinstance(remove, ImportError):
        remove = property(
            fget=lambda x: raise_(
                ImportError("Can't use XOutput method remove: " + str(remove))
            )
        )
    else:
        remove = remove
    # cf Methods.Output.XOutput.append
    if isinstance(append, ImportError):
        append = property(
            fget=lambda x: raise_(
                ImportError("Can't use XOutput method append: " + str(append))
            )
        )
    else:
        append = append
    # cf Methods.Output.XOutput.get_simu
    if isinstance(get_simu, ImportError):
        get_simu = property(
            fget=lambda x: raise_(
                ImportError("Can't use XOutput method get_simu: " + str(get_simu))
            )
        )
    else:
        get_simu = get_simu
    # cf Methods.Output.XOutput.keys
    if isinstance(keys, ImportError):
        keys = property(
            fget=lambda x: raise_(
                ImportError("Can't use XOutput method keys: " + str(keys))
            )
        )
    else:
        keys = keys
    # cf Methods.Output.XOutput.items
    if isinstance(items, ImportError):
        items = property(
            fget=lambda x: raise_(
                ImportError("Can't use XOutput method items: " + str(items))
            )
        )
    else:
        items = items
    # cf Methods.Output.XOutput.get_param_simu
    if isinstance(get_param_simu, ImportError):
        get_param_simu = property(
            fget=lambda x: raise_(
                ImportError(
                    "Can't use XOutput method get_param_simu: " + str(get_param_simu)
                )
            )
        )
    else:
        get_param_simu = get_param_simu
    # cf Methods.Output.XOutput.get_paramexplorer
    if isinstance(get_paramexplorer, ImportError):
        get_paramexplorer = property(
            fget=lambda x: raise_(
                ImportError(
                    "Can't use XOutput method get_paramexplorer: "
                    + str(get_paramexplorer)
                )
            )
        )
    else:
        get_paramexplorer = get_paramexplorer
    # cf Methods.Output.XOutput.plot_multi
    if isinstance(plot_multi, ImportError):
        plot_multi = property(
            fget=lambda x: raise_(
                ImportError("Can't use XOutput method plot_multi: " + str(plot_multi))
            )
        )
    else:
        plot_multi = plot_multi
    # cf Methods.Output.XOutput.plot_pareto
    if isinstance(plot_pareto, ImportError):
        plot_pareto = property(
            fget=lambda x: raise_(
                ImportError("Can't use XOutput method plot_pareto: " + str(plot_pareto))
            )
        )
    else:
        plot_pareto = plot_pareto
    # cf Methods.Output.XOutput.plot_generation
    if isinstance(plot_generation, ImportError):
        plot_generation = property(
            fget=lambda x: raise_(
                ImportError(
                    "Can't use XOutput method plot_generation: " + str(plot_generation)
                )
            )
        )
    else:
        plot_generation = plot_generation
    # cf Methods.Output.XOutput.get_pareto_index
    if isinstance(get_pareto_index, ImportError):
        get_pareto_index = property(
            fget=lambda x: raise_(
                ImportError(
                    "Can't use XOutput method get_pareto_index: "
                    + str(get_pareto_index)
                )
            )
        )
    else:
        get_pareto_index = get_pareto_index
    # cf Methods.Output.XOutput.print_memory
    if isinstance(print_memory, ImportError):
        print_memory = property(
            fget=lambda x: raise_(
                ImportError(
                    "Can't use XOutput method print_memory: " + str(print_memory)
                )
            )
        )
    else:
        print_memory = print_memory
    # cf Methods.Output.XOutput.get_xoutput_ref
    if isinstance(get_xoutput_ref, ImportError):
        get_xoutput_ref = property(
            fget=lambda x: raise_(
                ImportError(
                    "Can't use XOutput method get_xoutput_ref: " + str(get_xoutput_ref)
                )
            )
        )
    else:
        get_xoutput_ref = get_xoutput_ref
    # save and copy methods are available in all object
    save = save
    copy = copy
    # get_logger method is available in all object
    get_logger = get_logger

    def __init__(
        self,
        paramexplorer_list=-1,
        output_list=-1,
        xoutput_dict=-1,
        nb_simu=0,
        xoutput_ref=None,
        xoutput_ref_index=None,
        simu=-1,
        path_result="",
        geo=-1,
        elec=-1,
        mag=-1,
        struct=-1,
        post=-1,
        logger_name="Pyleecan.Output",
        force=-1,
        loss=-1,
        init_dict=None,
        init_str=None,
    ):
        """Constructor of the class. Can be use in three ways :
        - __init__ (arg1 = 1, arg3 = 5) every parameters have name and default values
            for pyleecan type, -1 will call the default constructor
        - __init__ (init_dict = d) d must be a dictionnary with property names as keys
        - __init__ (init_str = s) s must be a string
        s is the file path to load

        ndarray or list can be given for Vector and Matrix
        object or dict can be given for pyleecan Object"""

        if init_str is not None:  # Load from a file
            init_dict = load_init_dict(init_str)[1]
        if init_dict is not None:  # Initialisation by dict
            assert type(init_dict) is dict
            # Overwrite default value with init_dict content
            if "paramexplorer_list" in list(init_dict.keys()):
                paramexplorer_list = init_dict["paramexplorer_list"]
            if "output_list" in list(init_dict.keys()):
                output_list = init_dict["output_list"]
            if "xoutput_dict" in list(init_dict.keys()):
                xoutput_dict = init_dict["xoutput_dict"]
            if "nb_simu" in list(init_dict.keys()):
                nb_simu = init_dict["nb_simu"]
            if "xoutput_ref" in list(init_dict.keys()):
                xoutput_ref = init_dict["xoutput_ref"]
            if "xoutput_ref_index" in list(init_dict.keys()):
                xoutput_ref_index = init_dict["xoutput_ref_index"]
            if "simu" in list(init_dict.keys()):
                simu = init_dict["simu"]
            if "path_result" in list(init_dict.keys()):
                path_result = init_dict["path_result"]
            if "geo" in list(init_dict.keys()):
                geo = init_dict["geo"]
            if "elec" in list(init_dict.keys()):
                elec = init_dict["elec"]
            if "mag" in list(init_dict.keys()):
                mag = init_dict["mag"]
            if "struct" in list(init_dict.keys()):
                struct = init_dict["struct"]
            if "post" in list(init_dict.keys()):
                post = init_dict["post"]
            if "logger_name" in list(init_dict.keys()):
                logger_name = init_dict["logger_name"]
            if "force" in list(init_dict.keys()):
                force = init_dict["force"]
            if "loss" in list(init_dict.keys()):
                loss = init_dict["loss"]
        # Set the properties (value check and convertion are done in setter)
        self.paramexplorer_list = paramexplorer_list
        self.output_list = output_list
        self.xoutput_dict = xoutput_dict
        self.nb_simu = nb_simu
        self.xoutput_ref = xoutput_ref
        self.xoutput_ref_index = xoutput_ref_index
        # Call Output init
        super(XOutput, self).__init__(
            simu=simu,
            path_result=path_result,
            geo=geo,
            elec=elec,
            mag=mag,
            struct=struct,
            post=post,
            logger_name=logger_name,
            force=force,
            loss=loss,
        )
        # The class is frozen (in Output init), for now it's impossible to
        # add new properties

    def __str__(self):
        """Convert this object in a readeable string (for print)"""

        XOutput_str = ""
        # Get the properties inherited from Output
        XOutput_str += super(XOutput, self).__str__()
        if len(self.paramexplorer_list) == 0:
            XOutput_str += "paramexplorer_list = []" + linesep
        for ii in range(len(self.paramexplorer_list)):
            tmp = (
                self.paramexplorer_list[ii].__str__().replace(linesep, linesep + "\t")
                + linesep
            )
            XOutput_str += (
                "paramexplorer_list[" + str(ii) + "] =" + tmp + linesep + linesep
            )
        if len(self.output_list) == 0:
            XOutput_str += "output_list = []" + linesep
        for ii in range(len(self.output_list)):
            tmp = (
                self.output_list[ii].__str__().replace(linesep, linesep + "\t")
                + linesep
            )
            XOutput_str += "output_list[" + str(ii) + "] =" + tmp + linesep + linesep
        if len(self.xoutput_dict) == 0:
            XOutput_str += "xoutput_dict = dict()" + linesep
        for key, obj in self.xoutput_dict.items():
            tmp = (
                self.xoutput_dict[key].__str__().replace(linesep, linesep + "\t")
                + linesep
            )
            XOutput_str += "xoutput_dict[" + key + "] =" + tmp + linesep + linesep
        XOutput_str += "nb_simu = " + str(self.nb_simu) + linesep
        if self.xoutput_ref is not None:
            tmp = (
                self.xoutput_ref.__str__().replace(linesep, linesep + "\t").rstrip("\t")
            )
            XOutput_str += "xoutput_ref = " + tmp
        else:
            XOutput_str += "xoutput_ref = None" + linesep + linesep
        XOutput_str += "xoutput_ref_index = " + str(self.xoutput_ref_index) + linesep
        return XOutput_str

    def __eq__(self, other):
        """Compare two objects (skip parent)"""

        if type(other) != type(self):
            return False

        # Check the properties inherited from Output
        if not super(XOutput, self).__eq__(other):
            return False
        if other.paramexplorer_list != self.paramexplorer_list:
            return False
        if other.output_list != self.output_list:
            return False
        if other.xoutput_dict != self.xoutput_dict:
            return False
        if other.nb_simu != self.nb_simu:
            return False
        if other.xoutput_ref != self.xoutput_ref:
            return False
        if other.xoutput_ref_index != self.xoutput_ref_index:
            return False
        return True

    def compare(self, other, name="self"):
        """Compare two objects and return list of differences"""

        if type(other) != type(self):
            return ["type(" + name + ")"]
        diff_list = list()

        # Check the properties inherited from Output
        diff_list.extend(super(XOutput, self).compare(other, name=name))
        if (
            other.paramexplorer_list is None and self.paramexplorer_list is not None
        ) or (other.paramexplorer_list is not None and self.paramexplorer_list is None):
            diff_list.append(name + ".paramexplorer_list None mismatch")
        elif self.paramexplorer_list is None:
            pass
        elif len(other.paramexplorer_list) != len(self.paramexplorer_list):
            diff_list.append("len(" + name + ".paramexplorer_list)")
        else:
            for ii in range(len(other.paramexplorer_list)):
                diff_list.extend(
                    self.paramexplorer_list[ii].compare(
                        other.paramexplorer_list[ii],
                        name=name + ".paramexplorer_list[" + str(ii) + "]",
                    )
                )
        if (other.output_list is None and self.output_list is not None) or (
            other.output_list is not None and self.output_list is None
        ):
            diff_list.append(name + ".output_list None mismatch")
        elif self.output_list is None:
            pass
        elif len(other.output_list) != len(self.output_list):
            diff_list.append("len(" + name + ".output_list)")
        else:
            for ii in range(len(other.output_list)):
                diff_list.extend(
                    self.output_list[ii].compare(
                        other.output_list[ii],
                        name=name + ".output_list[" + str(ii) + "]",
                    )
                )
        if (other.xoutput_dict is None and self.xoutput_dict is not None) or (
            other.xoutput_dict is not None and self.xoutput_dict is None
        ):
            diff_list.append(name + ".xoutput_dict None mismatch")
        elif self.xoutput_dict is None:
            pass
        elif len(other.xoutput_dict) != len(self.xoutput_dict):
            diff_list.append("len(" + name + "xoutput_dict)")
        else:
            for key in self.xoutput_dict:
                diff_list.extend(
                    self.xoutput_dict[key].compare(
                        other.xoutput_dict[key], name=name + ".xoutput_dict"
                    )
                )
        if other._nb_simu != self._nb_simu:
            diff_list.append(name + ".nb_simu")
        if (other.xoutput_ref is None and self.xoutput_ref is not None) or (
            other.xoutput_ref is not None and self.xoutput_ref is None
        ):
            diff_list.append(name + ".xoutput_ref None mismatch")
        elif self.xoutput_ref is not None:
            diff_list.extend(
                self.xoutput_ref.compare(other.xoutput_ref, name=name + ".xoutput_ref")
            )
        if other._xoutput_ref_index != self._xoutput_ref_index:
            diff_list.append(name + ".xoutput_ref_index")
        return diff_list

    def __sizeof__(self):
        """Return the size in memory of the object (including all subobject)"""

        S = 0  # Full size of the object

        # Get size of the properties inherited from Output
        S += super(XOutput, self).__sizeof__()
        if self.paramexplorer_list is not None:
            for value in self.paramexplorer_list:
                S += getsizeof(value)
        if self.output_list is not None:
            for value in self.output_list:
                S += getsizeof(value)
        if self.xoutput_dict is not None:
            for key, value in self.xoutput_dict.items():
                S += getsizeof(value) + getsizeof(key)
        S += getsizeof(self.nb_simu)
        S += getsizeof(self.xoutput_ref)
        S += getsizeof(self.xoutput_ref_index)
        return S

    def as_dict(self, **kwargs):
        """
        Convert this object in a json serializable dict (can be use in __init__).
        Optional keyword input parameter is for internal use only
        and may prevent json serializability.
        """

        # Get the properties inherited from Output
        XOutput_dict = super(XOutput, self).as_dict(**kwargs)
        if self.paramexplorer_list is None:
            XOutput_dict["paramexplorer_list"] = None
        else:
            XOutput_dict["paramexplorer_list"] = list()
            for obj in self.paramexplorer_list:
                if obj is not None:
                    XOutput_dict["paramexplorer_list"].append(obj.as_dict(**kwargs))
                else:
                    XOutput_dict["paramexplorer_list"].append(None)
        if self.output_list is None:
            XOutput_dict["output_list"] = None
        else:
            XOutput_dict["output_list"] = list()
            for obj in self.output_list:
                if obj is not None:
                    XOutput_dict["output_list"].append(obj.as_dict(**kwargs))
                else:
                    XOutput_dict["output_list"].append(None)
        if self.xoutput_dict is None:
            XOutput_dict["xoutput_dict"] = None
        else:
            XOutput_dict["xoutput_dict"] = dict()
            for key, obj in self.xoutput_dict.items():
                if obj is not None:
                    XOutput_dict["xoutput_dict"][key] = obj.as_dict(**kwargs)
                else:
                    XOutput_dict["xoutput_dict"][key] = None
        XOutput_dict["nb_simu"] = self.nb_simu
        if self.xoutput_ref is None:
            XOutput_dict["xoutput_ref"] = None
        else:
            XOutput_dict["xoutput_ref"] = self.xoutput_ref.as_dict(**kwargs)
        XOutput_dict["xoutput_ref_index"] = self.xoutput_ref_index
        # The class name is added to the dict for deserialisation purpose
        # Overwrite the mother class name
        XOutput_dict["__class__"] = "XOutput"
        return XOutput_dict

    def _set_None(self):
        """Set all the properties to None (except pyleecan object)"""

        self.paramexplorer_list = None
        self.output_list = None
        self.xoutput_dict = None
        self.nb_simu = None
        if self.xoutput_ref is not None:
            self.xoutput_ref._set_None()
        self.xoutput_ref_index = None
        # Set to None the properties inherited from Output
        super(XOutput, self)._set_None()

    def _get_paramexplorer_list(self):
        """getter of paramexplorer_list"""
        if self._paramexplorer_list is not None:
            for obj in self._paramexplorer_list:
                if obj is not None:
                    obj.parent = self
        return self._paramexplorer_list

    def _set_paramexplorer_list(self, value):
        """setter of paramexplorer_list"""
        if type(value) is list:
            for ii, obj in enumerate(value):
                if type(obj) is dict:
                    class_obj = import_class(
                        "pyleecan.Classes", obj.get("__class__"), "paramexplorer_list"
                    )
                    value[ii] = class_obj(init_dict=obj)
                if value[ii] is not None:
                    value[ii].parent = self
        if value == -1:
            value = list()
        check_var("paramexplorer_list", value, "[ParamExplorer]")
        self._paramexplorer_list = value

    paramexplorer_list = property(
        fget=_get_paramexplorer_list,
        fset=_set_paramexplorer_list,
        doc=u"""List containing ParamExplorer

        :Type: [ParamExplorer]
        """,
    )

    def _get_output_list(self):
        """getter of output_list"""
        if self._output_list is not None:
            for obj in self._output_list:
                if obj is not None:
                    obj.parent = self
        return self._output_list

    def _set_output_list(self, value):
        """setter of output_list"""
        if type(value) is list:
            for ii, obj in enumerate(value):
                if type(obj) is dict:
                    class_obj = import_class(
                        "pyleecan.Classes", obj.get("__class__"), "output_list"
                    )
                    value[ii] = class_obj(init_dict=obj)
                if value[ii] is not None:
                    value[ii].parent = self
        if value == -1:
            value = list()
        check_var("output_list", value, "[Output]")
        self._output_list = value

    output_list = property(
        fget=_get_output_list,
        fset=_set_output_list,
        doc=u"""List containing Output (or Xoutput) for each simulation

        :Type: [Output]
        """,
    )

    def _get_xoutput_dict(self):
        """getter of xoutput_dict"""
        if self._xoutput_dict is not None:
            for key, obj in self._xoutput_dict.items():
                if obj is not None:
                    obj.parent = self
        return self._xoutput_dict

    def _set_xoutput_dict(self, value):
        """setter of xoutput_dict"""
        if type(value) is dict:
            for key, obj in value.items():
                if type(obj) is dict:
                    class_obj = import_class(
                        "pyleecan.Classes", obj.get("__class__"), "xoutput_dict"
                    )
                    value[key] = class_obj(init_dict=obj)
        if type(value) is int and value == -1:
            value = dict()
        check_var("xoutput_dict", value, "{DataKeeper}")
        self._xoutput_dict = value

    xoutput_dict = property(
        fget=_get_xoutput_dict,
        fset=_set_xoutput_dict,
        doc=u"""Dictionnary containing DataKeeper

        :Type: {DataKeeper}
        """,
    )

    def _get_nb_simu(self):
        """getter of nb_simu"""
        return self._nb_simu

    def _set_nb_simu(self, value):
        """setter of nb_simu"""
        check_var("nb_simu", value, "int", Vmin=0)
        self._nb_simu = value

    nb_simu = property(
        fget=_get_nb_simu,
        fset=_set_nb_simu,
        doc=u"""Number of simulations excluding reference simulation

        :Type: int
        :min: 0
        """,
    )

    def _get_xoutput_ref(self):
        """getter of xoutput_ref"""
        return self._xoutput_ref

    def _set_xoutput_ref(self, value):
        """setter of xoutput_ref"""
        if isinstance(value, str):  # Load from file
            value = load_init_dict(value)[1]
        if isinstance(value, dict) and "__class__" in value:
            class_obj = import_class(
                "pyleecan.Classes", value.get("__class__"), "xoutput_ref"
            )
            value = class_obj(init_dict=value)
        elif type(value) is int and value == -1:  # Default constructor
            value = Output()
        check_var("xoutput_ref", value, "Output")
        self._xoutput_ref = value

        if self._xoutput_ref is not None:
            self._xoutput_ref.parent = self

    xoutput_ref = property(
        fget=_get_xoutput_ref,
        fset=_set_xoutput_ref,
        doc=u"""Xoutput (or Output) of the reference simulation (only if is_keep_all_output is True and not included in output_list)

        :Type: Output
        """,
    )

    def _get_xoutput_ref_index(self):
        """getter of xoutput_ref_index"""
        return self._xoutput_ref_index

    def _set_xoutput_ref_index(self, value):
        """setter of xoutput_ref_index"""
        check_var("xoutput_ref_index", value, "int")
        self._xoutput_ref_index = value

    xoutput_ref_index = property(
        fget=_get_xoutput_ref_index,
        fset=_set_xoutput_ref_index,
        doc=u"""Index of the Xoutput (or Output) of the reference simulation in the output_list (only if is_keep_all_output is True)

        :Type: int
        """,
    )



# -*- coding: utf-8 -*-

import numpy as np


def shape_function(self, points):
    """ Return the values of linear shape functions in reference triangle for a given point """
    # TODO
    return None, None



# -*- coding: utf-8 -*-


def build_geometry(self, sym=1, alpha=0, delta=0, is_simplified=False):
    """Build geometry of the LamSquirrelCage

    Parameters
    ----------
    self :
        LamSquirrelCage Object
    sym : int
        Symmetry factor (1= full machine, 2= half of the machine...)
    alpha : float
        Angle for rotation [rad]
    delta : complex
        Complex value for translation
    is_simplified: bool
        True to avoid line superposition

    Returns
    -------
    surf_list: list
        list of surfaces

    """
    surf_list = super(type(self), self).build_geometry(
        sym=sym, is_simplified=is_simplified, alpha=alpha, delta=delta
    )

    # Adapt the label
    for surf in surf_list:
        if "Wind" in surf.label:
            surf.label = surf.label.replace("Wind", "Bar")

    return surf_list



from numpy import arcsin, exp

from ....Classes.Segment import Segment
from ....Classes.Arc1 import Arc1
from ....Classes.SurfLine import SurfLine


def get_surface_active(self, alpha=0, delta=0):
    """Return the full winding surface

    Parameters
    ----------
    self : SlotUD2
        A SlotUD2 object
    alpha : float
        float number for rotation (Default value = 0) [rad]
    delta : complex
        complex number for translation (Default value = 0)

    Returns
    -------
    surf_wind: Surface
        Surface corresponding to the Winding Area
    """
    st = self.get_name_lam()
    surface = self.active_surf.copy()

    surface.label = "Wind_" + st + "_R0_T0_S0"

    # Apply transformation
    surface.rotate(alpha)
    surface.translate(delta)

    return surface



# -*- coding: utf-8 -*-
"""File generated according to SWindParam/gen_list.json
WARNING! All changes made in this file will be lost!
"""
from pyleecan.GUI.Dialog.DMachineSetup.SWindParam.Ui_SWindParam import Ui_SWindParam


class Gen_SWindParam(Ui_SWindParam):
    def setupUi(self, SWindParam):
        """Abstract class to update the widget according to the csv doc"""
        Ui_SWindParam.setupUi(self, SWindParam)
        # Setup of in_Ntcoil
        txt = self.tr(u"""number of turns per coil""")
        self.in_Ntcoil.setWhatsThis(txt)
        self.in_Ntcoil.setToolTip(txt)

        # Setup of si_Ntcoil
        self.si_Ntcoil.setMinimum(1)
        self.si_Ntcoil.setMaximum(1000)
        txt = self.tr(u"""number of turns per coil""")
        self.si_Ntcoil.setWhatsThis(txt)
        self.si_Ntcoil.setToolTip(txt)

        # Setup of in_Npcpp
        txt = self.tr(u"""number of parallel circuits per phase (maximum 2p)""")
        self.in_Npcpp.setWhatsThis(txt)
        self.in_Npcpp.setToolTip(txt)

        # Setup of si_Npcpp
        self.si_Npcpp.setMinimum(1)
        self.si_Npcpp.setMaximum(1000)
        txt = self.tr(u"""number of parallel circuits per phase (maximum 2p)""")
        self.si_Npcpp.setWhatsThis(txt)
        self.si_Npcpp.setToolTip(txt)



# -*- coding: utf-8 -*-


def get_normals(self, indices=None, loc="center"):
    """Return the array of the normals coordinates.

    Parameters
    ----------
    self : MeshVTK
        a MeshVTK object
    indices : list
        list of the points to extract (optional)
    loc : str
        localization of the normals ("center" or "point")

    Returns
    -------
    normals: ndarray
        Normals coordinates
    """

    surf = self.get_surf(indices=indices)

    if loc == "center":
        return surf.cell_normals
    elif loc == "point":
        surf.compute_normals(cell_normals=False, point_normals=True, inplace=True)
        return surf["Normals"]



# -*- coding: utf-8 -*-
# File generated according to Generator/ClassesRef/Output/OutLoss.csv
# WARNING! All changes made in this file will be lost!
"""Method code available at https://github.com/Eomys/pyleecan/tree/master/pyleecan/Methods/Output/OutLoss
"""

from os import linesep
from sys import getsizeof
from logging import getLogger
from ._check import check_var, raise_
from ..Functions.get_logger import get_logger
from ..Functions.save import save
from ..Functions.copy import copy
from ..Functions.load import load_init_dict
from ..Functions.Load.import_class import import_class
from ._frozen import FrozenClass

# Import all class method
# Try/catch to remove unnecessary dependencies in unused method
try:
    from ..Methods.Output.OutLoss.get_loss import get_loss
except ImportError as error:
    get_loss = error

try:
    from ..Methods.Output.OutLoss.get_loss_dist import get_loss_dist
except ImportError as error:
    get_loss_dist = error


from ._check import InitUnKnowClassError
from .MeshSolution import MeshSolution


class OutLoss(FrozenClass):
    """Gather the loss module outputs"""

    VERSION = 1

    # Check ImportError to remove unnecessary dependencies in unused method
    # cf Methods.Output.OutLoss.get_loss
    if isinstance(get_loss, ImportError):
        get_loss = property(
            fget=lambda x: raise_(
                ImportError("Can't use OutLoss method get_loss: " + str(get_loss))
            )
        )
    else:
        get_loss = get_loss
    # cf Methods.Output.OutLoss.get_loss_dist
    if isinstance(get_loss_dist, ImportError):
        get_loss_dist = property(
            fget=lambda x: raise_(
                ImportError(
                    "Can't use OutLoss method get_loss_dist: " + str(get_loss_dist)
                )
            )
        )
    else:
        get_loss_dist = get_loss_dist
    # save and copy methods are available in all object
    save = save
    copy = copy
    # get_logger method is available in all object
    get_logger = get_logger

    def __init__(
        self,
        loss_list=None,
        meshsol_list=-1,
        loss_index=-1,
        logger_name="Pyleecan.Loss",
        init_dict=None,
        init_str=None,
    ):
        """Constructor of the class. Can be use in three ways :
        - __init__ (arg1 = 1, arg3 = 5) every parameters have name and default values
            for pyleecan type, -1 will call the default constructor
        - __init__ (init_dict = d) d must be a dictionnary with property names as keys
        - __init__ (init_str = s) s must be a string
        s is the file path to load

        ndarray or list can be given for Vector and Matrix
        object or dict can be given for pyleecan Object"""

        if init_str is not None:  # Load from a file
            init_dict = load_init_dict(init_str)[1]
        if init_dict is not None:  # Initialisation by dict
            assert type(init_dict) is dict
            # Overwrite default value with init_dict content
            if "loss_list" in list(init_dict.keys()):
                loss_list = init_dict["loss_list"]
            if "meshsol_list" in list(init_dict.keys()):
                meshsol_list = init_dict["meshsol_list"]
            if "loss_index" in list(init_dict.keys()):
                loss_index = init_dict["loss_index"]
            if "logger_name" in list(init_dict.keys()):
                logger_name = init_dict["logger_name"]
        # Set the properties (value check and convertion are done in setter)
        self.parent = None
        self.loss_list = loss_list
        self.meshsol_list = meshsol_list
        self.loss_index = loss_index
        self.logger_name = logger_name

        # The class is frozen, for now it's impossible to add new properties
        self._freeze()

    def __str__(self):
        """Convert this object in a readeable string (for print)"""

        OutLoss_str = ""
        if self.parent is None:
            OutLoss_str += "parent = None " + linesep
        else:
            OutLoss_str += "parent = " + str(type(self.parent)) + " object" + linesep
        OutLoss_str += "loss_list = " + str(self.loss_list) + linesep + linesep
        if len(self.meshsol_list) == 0:
            OutLoss_str += "meshsol_list = []" + linesep
        for ii in range(len(self.meshsol_list)):
            tmp = (
                self.meshsol_list[ii].__str__().replace(linesep, linesep + "\t")
                + linesep
            )
            OutLoss_str += "meshsol_list[" + str(ii) + "] =" + tmp + linesep + linesep
        OutLoss_str += "loss_index = " + str(self.loss_index) + linesep
        OutLoss_str += 'logger_name = "' + str(self.logger_name) + '"' + linesep
        return OutLoss_str

    def __eq__(self, other):
        """Compare two objects (skip parent)"""

        if type(other) != type(self):
            return False
        if other.loss_list != self.loss_list:
            return False
        if other.meshsol_list != self.meshsol_list:
            return False
        if other.loss_index != self.loss_index:
            return False
        if other.logger_name != self.logger_name:
            return False
        return True

    def compare(self, other, name="self"):
        """Compare two objects and return list of differences"""

        if type(other) != type(self):
            return ["type(" + name + ")"]
        diff_list = list()
        if (other.loss_list is None and self.loss_list is not None) or (
            other.loss_list is not None and self.loss_list is None
        ):
            diff_list.append(name + ".loss_list None mismatch")
        elif self.loss_list is None:
            pass
        elif len(other.loss_list) != len(self.loss_list):
            diff_list.append("len(" + name + ".loss_list)")
        else:
            for ii in range(len(other.loss_list)):
                diff_list.extend(
                    self.loss_list[ii].compare(
                        other.loss_list[ii], name=name + ".loss_list[" + str(ii) + "]"
                    )
                )
        if (other.meshsol_list is None and self.meshsol_list is not None) or (
            other.meshsol_list is not None and self.meshsol_list is None
        ):
            diff_list.append(name + ".meshsol_list None mismatch")
        elif self.meshsol_list is None:
            pass
        elif len(other.meshsol_list) != len(self.meshsol_list):
            diff_list.append("len(" + name + ".meshsol_list)")
        else:
            for ii in range(len(other.meshsol_list)):
                diff_list.extend(
                    self.meshsol_list[ii].compare(
                        other.meshsol_list[ii],
                        name=name + ".meshsol_list[" + str(ii) + "]",
                    )
                )
        if other._loss_index != self._loss_index:
            diff_list.append(name + ".loss_index")
        if other._logger_name != self._logger_name:
            diff_list.append(name + ".logger_name")
        return diff_list

    def __sizeof__(self):
        """Return the size in memory of the object (including all subobject)"""

        S = 0  # Full size of the object
        if self.loss_list is not None:
            for value in self.loss_list:
                S += getsizeof(value)
        if self.meshsol_list is not None:
            for value in self.meshsol_list:
                S += getsizeof(value)
        if self.loss_index is not None:
            for key, value in self.loss_index.items():
                S += getsizeof(value) + getsizeof(key)
        S += getsizeof(self.logger_name)
        return S

    def as_dict(self, **kwargs):
        """
        Convert this object in a json serializable dict (can be use in __init__).
        Optional keyword input parameter is for internal use only
        and may prevent json serializability.
        """

        OutLoss_dict = dict()
        if self.loss_list is None:
            OutLoss_dict["loss_list"] = None
        else:
            OutLoss_dict["loss_list"] = list()
            for obj in self.loss_list:
                if obj is not None:
                    OutLoss_dict["loss_list"].append(obj.as_dict())
                else:
                    OutLoss_dict["loss_list"].append(None)
        if self.meshsol_list is None:
            OutLoss_dict["meshsol_list"] = None
        else:
            OutLoss_dict["meshsol_list"] = list()
            for obj in self.meshsol_list:
                if obj is not None:
                    OutLoss_dict["meshsol_list"].append(obj.as_dict(**kwargs))
                else:
                    OutLoss_dict["meshsol_list"].append(None)
        OutLoss_dict["loss_index"] = (
            self.loss_index.copy() if self.loss_index is not None else None
        )
        OutLoss_dict["logger_name"] = self.logger_name
        # The class name is added to the dict for deserialisation purpose
        OutLoss_dict["__class__"] = "OutLoss"
        return OutLoss_dict

    def _set_None(self):
        """Set all the properties to None (except pyleecan object)"""

        self.loss_list = None
        self.meshsol_list = None
        self.loss_index = None
        self.logger_name = None

    def _get_loss_list(self):
        """getter of loss_list"""
        if self._loss_list is not None:
            for obj in self._loss_list:
                if obj is not None:
                    obj.parent = self
        return self._loss_list

    def _set_loss_list(self, value):
        """setter of loss_list"""
        if type(value) is list:
            for ii, obj in enumerate(value):
                if type(obj) is dict:
                    class_obj = import_class(
                        "SciDataTool.Classes", obj.get("__class__"), "loss_list"
                    )
                    value[ii] = class_obj(init_dict=obj)
                if value[ii] is not None:
                    value[ii].parent = self
        if value == -1:
            value = list()
        check_var("loss_list", value, "[DataND]")
        self._loss_list = value

    loss_list = property(
        fget=_get_loss_list,
        fset=_set_loss_list,
        doc=u"""Internal list of loss data

        :Type: [SciDataTool.Classes.DataND.DataND]
        """,
    )

    def _get_meshsol_list(self):
        """getter of meshsol_list"""
        if self._meshsol_list is not None:
            for obj in self._meshsol_list:
                if obj is not None:
                    obj.parent = self
        return self._meshsol_list

    def _set_meshsol_list(self, value):
        """setter of meshsol_list"""
        if type(value) is list:
            for ii, obj in enumerate(value):
                if type(obj) is dict:
                    class_obj = import_class(
                        "pyleecan.Classes", obj.get("__class__"), "meshsol_list"
                    )
                    value[ii] = class_obj(init_dict=obj)
                if value[ii] is not None:
                    value[ii].parent = self
        if value == -1:
            value = list()
        check_var("meshsol_list", value, "[MeshSolution]")
        self._meshsol_list = value

    meshsol_list = property(
        fget=_get_meshsol_list,
        fset=_set_meshsol_list,
        doc=u"""Internal list of loss meshsolutions

        :Type: [MeshSolution]
        """,
    )

    def _get_loss_index(self):
        """getter of loss_index"""
        return self._loss_index

    def _set_loss_index(self, value):
        """setter of loss_index"""
        if type(value) is int and value == -1:
            value = dict()
        check_var("loss_index", value, "dict")
        self._loss_index = value

    loss_index = property(
        fget=_get_loss_index,
        fset=_set_loss_index,
        doc=u"""Internal dict to index losses

        :Type: dict
        """,
    )

    def _get_logger_name(self):
        """getter of logger_name"""
        return self._logger_name

    def _set_logger_name(self, value):
        """setter of logger_name"""
        check_var("logger_name", value, "str")
        self._logger_name = value

    logger_name = property(
        fget=_get_logger_name,
        fset=_set_logger_name,
        doc=u"""Name of the logger to use

        :Type: str
        """,
    )



# -*- coding: utf-8 -*-

from numpy import pi, exp

from ....Classes.Arc1 import Arc1
from ....Classes.Arc3 import Arc3
from ....Classes.Segment import Segment
from ....Classes.SurfLine import SurfLine
from ....Functions.Geometry.inter_line_circle import inter_line_circle


def build_geometry(self, alpha=0, delta=0, is_simplified=False):
    """Compute the curve (Arc) needed to plot the Hole.
    The ending point of a curve is the starting point of the next curve in the
    list

    Parameters
    ----------
    self : HoleM54
        A HoleM54 object
    alpha : float
        Angle to rotate the slot (Default value = 0) [rad]
    delta : complex
        Complex to translate the slot (Default value = 0)
    is_simplified : bool
       True to avoid line superposition

    Returns
    -------
    surf_list : list
        List of Air Surface on the slot

    """

    Rext = self.get_Rext()
    # Get all the points
    point_dict = self._comp_point_coordinate()
    Z1 = point_dict["Z1"]
    Z2 = point_dict["Z2"]
    Z3 = point_dict["Z3"]
    Z4 = point_dict["Z4"]

    surf_list = list()
    curve_list = list()
    curve_list.append(Arc1(begin=Z1, end=Z2, radius=-self.R1, is_trigo_direction=False))
    curve_list.append(Arc3(begin=Z2, end=Z3, is_trigo_direction=True))
    curve_list.append(Arc1(begin=Z3, end=Z4, radius=self.R1 + self.H1))
    curve_list.append(Arc3(begin=Z4, end=Z1, is_trigo_direction=True))

    if self.get_is_stator():  # check if the slot is on the stator
        st = "_Stator"
    else:
        st = "_Rotor"
    Zref = Rext - self.H0 - self.H1 / 2
    surf_list.append(
        SurfLine(line_list=curve_list, label="Hole" + st + "_R0_T0_S0", point_ref=Zref)
    )

    # Apply the transformations
    for surf in surf_list:
        surf.rotate(alpha)
        surf.translate(delta)

    return surf_list



from numpy import exp, angle, abs as np_abs

DELTA = 1e-9  # To remove computing noise


def split_line(self, Z1, Z2, is_top=True, is_join=False, label_join=""):
    """Cut the Segment according to a line defined by two complex

    Parameters
    ----------
    self : Segment
        An Segment object
    Z1 : complex
        First point of the cutting Line
    Z2 : complex
        Second point of the cutting Line
    is_top : bool
        True to keep the part above the cutting line.
        "Above" is in the coordinate system with Z1 in 0 and Z2 on the X>0 axis
    is_join : bool
        True to join the split_list with Segment if there is more that one remaining parts
    label_join : str
        Label of the join line

    Returns
    -------
    split_list : list(Segment)
        The selected part of the Segment (0 or 1 segment depending on cutting point)
    """

    Z_int = self.intersect_line(Z1, Z2)
    # Begin and end point in the line coordonate system
    Zb = (self.begin - Z1) * exp(-1j * angle(Z2 - Z1))
    Ze = (self.end - Z1) * exp(-1j * angle(Z2 - Z1))

    if len(Z_int) == 0:
        # No intersection copy the line
        line = type(self)(init_dict=self.as_dict())
        if Zb.imag >= 0 and Ze.imag >= 0 and is_top:
            return [line]
        if Zb.imag >= 0 and Ze.imag >= 0 and not is_top:
            return []
        if Zb.imag <= 0 and Ze.imag <= 0 and is_top:
            return []
        return [line]
    if len(Z_int) == 1:
        # One intersection => Three possible lines
        # Begin => Intersection
        if np_abs(Z_int[0] - self.begin) > DELTA:
            line_list_1 = [type(self)(init_dict=self.as_dict())]
            line_list_1[0].end = Z_int[0]
        else:  # Begin == Intersection
            line_list_1 = []
        # Intersection => End
        if np_abs(Z_int[0] - self.end) > DELTA:
            line_list_2 = [type(self)(init_dict=self.as_dict())]
            line_list_2[0].begin = Z_int[0]
        else:  # Intersection == End
            line_list_2 = []
        # Copy of the complete line (begin or end on cutting line)
        line_list_3 = [type(self)(init_dict=self.as_dict())]

        # Return the correct line according to the points position
        if Zb.imag > DELTA and is_top:
            return line_list_1
        if Zb.imag > DELTA and not is_top:
            return line_list_2
        if Zb.imag < -DELTA and is_top:
            return line_list_2
        if Zb.imag < -DELTA and not is_top:
            return line_list_1
        # Zb.imag == 0 => begin on cutting line
        if Ze.imag > DELTA and is_top:
            return line_list_3
        if Ze.imag > DELTA and not is_top:
            return []
        if Ze.imag < -DELTA and is_top:
            return []
        if Ze.imag < -DELTA and not is_top:
            return line_list_3
    if len(Z_int) == 2:
        # The segment is on the line => Copy the line
        return [type(self)(init_dict=self.as_dict())]



# -*- coding: utf-8 -*-


def is_synchronous(self):
    """Return if a machine is synchronous or not

    Parameters
    ----------
    self : MachineUD
        A MachineUD

    Returns
    -------
    is_synchronous: bool
        User Defined machine are synchronous according to is_sync

    """
    return self.is_sync



# -*- coding: utf-8 -*-

from ....Methods.Slot.Slot import SlotCheckError
from ....Methods.Slot.HoleM52 import *


def check(self):
    """Check that the HoleM52 object is correct

    Parameters
    ----------
    self : HoleM52
        A HoleM52 object

    Returns
    -------
    None

    Raises
    -------
    S52_H12CheckError
        You must have H2 < H1
    S52_alphaCheckError
        The teeth are too wide for the lamination (reduce W3 or H0)
    S52_W1CheckError
        W1 is <=0, you must reduce W0 or W3
    """

    # Check that everything is set
    if self.W0 is None:
        raise S52_NoneError("You must set W0 !")
    elif self.W3 is None:
        raise S52_NoneError("You must set W3 !")
    elif self.H0 is None:
        raise S52_NoneError("You must set H0 !")
    elif self.H1 is None:
        raise S52_NoneError("You must set H1 !")
    elif self.H2 is None:
        raise S52_NoneError("You must set H2 !")

    if self.H2 >= self.H1:
        raise S52_H12CheckError("You must have H2 < H1")

    alpha = self.comp_alpha()
    if alpha <= 0:
        raise S52_alphaCheckError(
            "The teeth are too wide for the lamination (reduce W3 or H0)"
        )

    W1 = self.comp_W1()
    if W1 <= 0:
        raise S52_W1CheckError("W1 is <=0, you must reduce W0 or W3")



from ....Methods.Slot.Slot import SlotCheckError


class S50_NoneError(SlotCheckError):
    """Raised when a propery of HoleM50 is None"""

    pass


class S50_W01CheckError(SlotCheckError):
    """ """

    pass


class S50_H23CheckError(SlotCheckError):
    """ """

    pass


class S50_H01CheckError(SlotCheckError):
    """ """

    pass


class S50_W5CheckError(SlotCheckError):
    """ """

    pass


class S50_SpCheckError(SlotCheckError):
    """ """

    pass



# -*- coding: utf-8 -*-

from numpy import vstack, column_stack
from ....Methods.Import.ImportGenMatrixSin import (
    GenSinEmptyError,
    GenSinDimError,
    GenSinTransposeError,
)


def get_data(self):
    """Generate the sinus matrix

    Parameters
    ----------
    self : ImportGenMatrixSin
        An ImportGenMatrixSin object

    Returns
    -------
    matrix: ndarray
        The generated sinus matrix

    """

    if len(self.sin_list) == 0:
        raise GenSinEmptyError("ERROR: The sinus list is empty")

    N = self.sin_list[0].N
    is_T = self.sin_list[0].is_transpose

    for ii in range(1, len(self.sin_list)):
        if self.sin_list[ii].N != N:
            raise GenSinDimError(
                "ERROR: Dimension mismatch between ImportGenVectSin 0 (N="
                + str(N)
                + ") and ImportGenVectSin "
                + str(ii)
                + " (N="
                + str(self.sin_list[ii].N)
                + ")"
            )
        if self.sin_list[ii].is_transpose != is_T:
            raise GenSinTransposeError(
                "ERROR: is_transpose mismatch between ImportGenVectSin 0 ("
                + str(is_T)
                + ") and ImportGenVectSin "
                + str(ii)
                + " ("
                + str(self.sin_list[ii].is_transpose)
                + ")"
            )
    if is_T:  # Conctenate the columns
        matrix = column_stack([sin_obj.get_data() for sin_obj in self.sin_list])
    else:  # Concatenate the lines
        matrix = vstack([sin_obj.get_data() for sin_obj in self.sin_list])
    return self.edit_matrix(matrix)



# -*- coding: utf-8 -*-
from matplotlib.patches import Circle

from ....definitions import config_dict

PATCH_EDGE = config_dict["PLOT"]["COLOR_DICT"]["PATCH_EDGE"]
PATCH_COLOR = config_dict["PLOT"]["COLOR_DICT"]["PATCH_COLOR"]
PATCH_COLOR_ALPHA = config_dict["PLOT"]["COLOR_DICT"]["PATCH_COLOR_ALPHA"]
PATCH_EDGE_ALPHA = config_dict["PLOT"]["COLOR_DICT"]["PATCH_EDGE_ALPHA"]


def get_patches(
    self, color=PATCH_COLOR, edgecolor=PATCH_EDGE, is_edge_only=False, linestyle=None
):
    """Returns the Circle Patch to be display in matplotlib

    Parameters
    ----------
    self : Circle
          a Circle object
    color :
        the color of the Patch (Default value = PATCH_COLOR)
    edgecolor :
        edgecolor of the Patch (Default value = PATCH_EDGE)
    is_edge_only: bool
        To set the transparancy of the face color to 0 and 1 for the edge color
    linestyle : str
        Line style of the edge {'-', '--', '-.', ':', '', (offset, on-off-seq), ...}

    Returns
    -------
    patch_list : list of matplotlib.patches.Circle
        List of patches corresponding to the surface
    """

    # check if the Circle is correct
    self.check()
    # the coordinates of the center of the circle
    Zr = self.center.real
    Zi = self.center.imag

    if is_edge_only:
        color = PATCH_COLOR_ALPHA
        edgecolor = PATCH_EDGE_ALPHA

    return [
        Circle(
            xy=(Zr, Zi),
            radius=self.radius,
            facecolor=color,
            edgecolor=edgecolor,
            linestyle=linestyle,
        )
    ]



from ....Functions.Load.import_class import import_class


def convert_to_UD(self):
    """Convert the hole to HoleUD"""
    HoleUD = import_class("pyleecan.Classes", "HoleUD")
    surf_list = self.build_geometry()
    magnet_dict = self.get_magnet_dict()
    magnetization_dict = self.comp_magnetization_dict()

    return HoleUD(
        surf_list=surf_list,
        magnet_dict=magnet_dict,
        Zh=self.Zh,
        mat_void=self.mat_void,
        magnetization_dict_offset=magnetization_dict,
    )



# -*- coding: utf-8 -*-
from ....Classes.Output import Output
from deap import base, creator, tools


def delete_toolbox(self):
    """OptiGenAlgNsga2Deap method to delete DEAP toolbox
    Parameters
    ----------
    self : OptiGenAlgNsga2Deap
    """

    # Delete toolbox
    self.toolbox = None

    # Delete creator classes
    del creator.FitnessMin
    del creator.Individual



# -*- coding: utf-8 -*-

from numpy import array


def comp_radius_mec(self):
    """Compute the mechanical radius of the Lamination [m]

    Parameters
    ----------
    self : LamSlotMag
        A LamSlotMag object

    Returns
    -------
    Rmec: float
        Mechanical radius [m]

    """

    (Rmin, Rmax) = self.slot.comp_radius()

    if self.is_internal:  # inward Slot
        # Top radius of the magnet
        return max(self.Rext, Rmax)
    else:
        return min(self.Rint, Rmin)



# -*- coding: utf-8 -*-

from numpy import exp, angle, pi


def get_angle(self, is_deg=False):
    """Return the angle of the arc

    Parameters
    ----------
    self : Arc3
        An Arc3 object
    is_deg: bool
        True to convert to degree

    Returns
    -------
    angle: float
        Angle of the arc
    """

    if self.is_trigo_direction:
        sign = 1
    else:
        sign = -1

    if is_deg:
        return sign * 180
    else:
        return sign * pi



from numpy import arcsin, exp, cos


def _comp_point_coordinate(self):
    """Compute the point coordinates needed to plot the Slot.

    Parameters
    ----------
    self : SlotM15
        A SlotM15 object

    Returns
    -------
    point_dict: dict
        A dict of the slot coordinates
    """

    Rbo = self.get_Rbo()

    Z1 = Rbo * exp(-1j * self.W0 / 2)

    alpha_lim = 2 * arcsin(self.Wmag * 0.5 / self.Rtopm)  # Angle of the top arc
    H_top_arc = self.Rtopm * (1 - cos(alpha_lim / 2))  # Heiht of the top arc
    if self.is_outwards():
        Z2 = (Rbo + self.H0) * exp(-1j * self.W0 / 2)
    else:  # inward slot
        Z2 = (Rbo - self.H0) * exp(-1j * self.W0 / 2)

    alpha_M = float(arcsin(self.Wmag / (2 * abs(Z2))))
    ZM1 = abs(Z2) * exp(-1j * alpha_M)
    if self.is_outwards():
        ZM0 = abs(Z2) - self.Hmag
        Zc = ZM0 + self.Rtopm
    else:  # inward slot
        ZM0 = abs(Z2) + self.Hmag
        Zc = ZM0 - self.Rtopm

    sign = -1 if self.is_outwards() else 1
    x = abs(Z2) - sign * (
        -self.Hmag + self.Rtopm - (self.Rtopm ** 2 - self.Wmag ** 2 / 4) ** 0.5
    )
    ZM2 = x - 1j * self.Wmag / 2

    point_dict = dict()
    point_dict["Z1"] = Z1
    point_dict["Z2"] = Z2
    point_dict["ZM1"] = ZM1
    point_dict["ZM2"] = ZM2
    point_dict["ZM0"] = ZM0
    point_dict["Zc"] = Zc
    # symetry
    point_dict["Z3"] = Z2.conjugate()
    point_dict["Z4"] = Z1.conjugate()
    point_dict["ZM3"] = ZM2.conjugate()
    point_dict["ZM4"] = ZM1.conjugate()

    return point_dict



# -*- coding: utf-8 -*-

from numpy import abs as np_abs


def get_center(self):
    """Return the center of the arc

    Parameters
    ----------
    self : Arc3
        An Arc3 object

    Returns
    -------
    Zc: complex
        Complex coordinates of the center of the Arc3
    """

    self.check()

    # the center is on the bisection of [begin, end]
    z1 = self.begin
    z2 = self.end

    Zc = (z1 + z2) / 2.0

    # Return (0,0) if the point is too close from 0
    if np_abs(Zc) < 1e-6:
        Zc = 0

    return Zc



# -*- coding: utf-8 -*-

from numpy import pi, angle, exp

from ....Classes.Circle import Circle
from ....Classes.SurfLine import SurfLine
from ....Classes.Arc1 import Arc1
from ....Classes.Segment import Segment


def get_Zs(self):
    """Return the number of Slot of the Lamination

    Parameters
    ----------
    self : LamSlotMulti
        a LamSlotMulti object

    Returns
    -------
    Zs : float
        Number of Slot

    """

    return len(self.slot_list)



from numpy import sin


def comp_surface(self):
    """Compute the Slot total surface (by analytical computation).
    Caution, the top of the Slot is an Arc

    Parameters
    ----------
    self : SlotDC
        A SlotDC object

    Returns
    -------
    S: float
        Slot total surface [m**2]

    """

    [
        Z1,
        Z2,
        Z3,
        Z4,
        Z5,
        Z6,
        Z7,
        Z8,
        Z9,
        Z10,
        Z11,
        Z12,
        _,
        _,
        _,
    ] = self._comp_point_coordinate()

    Rbo = self.get_Rbo()

    Swind = self.comp_surface_active()
    # Rectangle Z12,Z11,Z2,Z1
    S6 = self.W1 * abs(Z12.real - Z11.real)
    # Top arc of the slot (on bore radius)
    alpha = self.comp_angle_opening()
    Sarc = (Rbo ** 2.0) / 2.0 * (alpha - sin(alpha))

    if self.is_outwards():
        return Swind + S6 - Sarc
    else:
        return Swind + S6 + Sarc



from numpy import pi


def comp_angle_d_axis(self):
    """Compute the angle between the X axis and the first d+ axis
    By convention a "Tooth" is centered on the X axis
    By convention the first magnet is +

    Parameters
    ----------
    self : LamHole
        A LamHole object

    Returns
    -------
    d_angle : float
        angle between the X axis and the first d+ axis
    """

    p = self.get_pole_pair_number()
    if self.has_magnet():
        return pi / p / 2
    else:
        return 0



def comp_height(self):
    """Compute the height of the Slot.
    Caution, the bottom of the Slot is an Arc

    Parameters
    ----------
    self : SlotW16
        A SlotW16 object

    Returns
    -------
    Htot: float
        Height of the slot [m]

    """

    return self.H0 + self.H2



from numpy import arcsin, exp, pi


def _comp_point_coordinate(self):
    """Compute the point coordinates needed to plot the Slot.

    Parameters
    ----------
    self : SlotW24
        A SlotW24 object

    Returns
    -------
    point_dict: dict
        A dict of the slot point coordinates
    """
    Rbo = self.get_Rbo()

    # alpha_0 is the angle to have a W3 tooth width on the bore
    alpha_0 = float(arcsin(self.W3 / (2 * Rbo)))

    # comp point coordinate (in complex)
    Z0 = Rbo * exp(1j * 0)
    Z1t = Z0 * exp(1j * alpha_0)

    if self.is_outwards():
        Z2t = Z1t + self.H2
    else:  # inward slot
        Z2t = Z1t - self.H2
    hsp = pi / self.Zs

    point_dict = dict()
    # symetry
    point_dict["Z1"] = Z1t * exp(-1j * hsp)
    point_dict["Z2"] = Z2t * exp(-1j * hsp)
    point_dict["Z3"] = point_dict["Z2"].conjugate()
    point_dict["Z4"] = point_dict["Z1"].conjugate()
    return point_dict



import numpy as np
import h5py
from h5py import File as FileH5
from ...definitions import PACKAGE_NAME
from ... import __version__
from datetime import datetime


def pyleecan_dict_to_hdf5(file, obj):
    """
    Save a dict from a pyleecan object in the hdf5 file

    Parameters
    ----------
    file: hdf5 file
    obj: Pyleecan object
        object to save
    """
    obj_dict = obj.as_dict()
    now = datetime.now()
    obj_dict["__save_date__"] = now.strftime("%Y_%m_%d %Hh%Mmin%Ss ")
    obj_dict["__version__"] = PACKAGE_NAME + "_" + __version__
    for key, val in obj_dict.items():
        # Object that need groups
        if isinstance(val, dict) or isinstance(val, list):
            variable_to_hdf5(file, "", val, key)
        # Dataset
        elif val == None:
            file[key] = "NoneValue"
        else:
            file[key] = val


def list_to_hdf5(file, group_name, name, list_to_save):
    """
    Save a list in the hdf5 file : save it as a ndarray if possible

    Parameters
    ----------
    file: HDF5 file
        file to save the data
    group_name: str
        name of the group
    name: str
        name to extend the group or to contain the dataset
    list_to_save: list
        list to save

    """
    #

    # Convert into array
    array_list = np.array(list_to_save)

    # Check the type to split or save as an array
    if array_list.dtype.kind in ["O", "U"]:
        # Split the list into several datasets
        # Create a new group to contain every dataset
        group_name = group_name + "/" + name
        grp = file.create_group(group_name)

        # Add an attribute to load correctly
        grp.attrs["length_list"] = len(list_to_save)

        # Save every element of the list
        for i, element in enumerate(list_to_save):
            variable_to_hdf5(file, group_name, element, "list_{}".format(i))

    else:  # Save as an array
        if group_name == "":
            file[name] = array_list
            file[name].attrs["array_list"] = True
        else:
            grp = file[group_name]
            grp[name] = array_list
            # Add an attribute to load correctly
            grp[name].attrs["array_list"] = True


def dict_to_hdf5(file, prefix, dict_to_save):
    """
    Save a list in the hdf5 file
    """
    for key, value in dict_to_save.items():
        if isinstance(key, int):
            key = str(key)
        variable_to_hdf5(file, prefix, value, key)


def variable_to_hdf5(file, prefix, variable, name):
    # Pyleecan object dict
    if isinstance(variable, dict):
        # Create group
        group_name = prefix + "/" + name
        file.create_group(group_name)

        # Call function to create groups and datasets recursively
        dict_to_hdf5(file, group_name, variable)

    # List
    elif isinstance(variable, list):
        # Create group

        # Call function to create groups and datasets recursively
        list_to_hdf5(file, prefix, name, variable)
    # Str
    elif isinstance(variable, str):
        if len(variable) == 0:
            grp = file[prefix]
            grp[name] = variable
        else:
            grp = file[prefix]
            # Create a fixed-width ASCII string according
            # to http://docs.h5py.org/en/stable/strings.html#exceptions-for-python-3
            grp[name] = np.string_(variable.encode("ISO-8859-2"))
    # None
    elif variable is None:
        # Create dataset
        grp = file[prefix]
        grp[name] = "NoneValue"
    else:
        # Create dataset
        grp = file[prefix]
        grp[name] = variable


def save_hdf5(obj, save_path):
    """
    Save a pyleecan obj in hdf5 format

    Parameters
    ----------
    obj: Pyleecan object
        object to save
    save_path: str
        file path
    """

    file5 = None
    try:
        file5 = FileH5(save_path, "w")
        pyleecan_dict_to_hdf5(file5, obj)
        file5.close()
    except Exception as err:
        if file5:
            file5.close()
        raise (err)



import numpy as np
import matplotlib.pyplot as plt
from ....Classes.OptiObjective import OptiObjective


def get_pareto_index(self):
    """Return index of individuals in the pareto

    Parameters
    ----------
    self: XOutput

    Returns
    -------
    idx_non_dom: list
        list of index of non dominated individuals
    """

    # Gather fitness results
    data = [
        val.result
        for _, val in self.xoutput_dict.items()
        if isinstance(val, OptiObjective)
    ]
    fitness = np.array(data).T

    # Get fitness values and ngen
    is_valid = np.array(self["is_valid"])

    # Keep only valid values
    indx = np.where(is_valid)
    fitness = fitness[indx]

    # Get pareto front
    pareto = fitness

    # Get dominated values
    idx_non_dom = list(range(len(pareto)))
    N = len(pareto)
    for i in range(N):
        for j in idx_non_dom:
            if all(pareto[j] <= pareto[i]) and any(pareto[j] < pareto[i]):
                idx_non_dom.remove(i)
                break

    return idx_non_dom



# -*- coding: utf-8 -*-

import os
from os.path import join, isdir


def get_path_save_fem(self, output):
    """Return the path to save the FEMM simulation

    Parameters
    ----------
    self : MagFEMM
        a MagFEMM object
    output : Output
        an Output object

        Returns
    -------
    save_path: str
        Path to save the .fem file

    """
    save_dir = self.get_path_save(output)

    if self.file_name not in [None, ""]:
        if self.file_name[-4:] != ".fem":
            file_name = self.file_name + ".fem"
        else:
            file_name = self.file_name
    elif output.simu.machine.name not in [None, ""]:
        file_name = output.simu.machine.name + "_model.fem"
    elif output.simu.name not in [None, ""]:
        file_name = output.simu.name + "_model.fem"
    else:  # Default name
        file_name = "FEMM_simulation.fem"

    return join(save_dir, file_name)



from numpy import pi, exp


def _comp_point_coordinate(self):
    """Compute the point coordinates needed to plot the Slot.

    Parameters
    ----------
    self : HoleM54
        A HoleM54 object

    Returns
    -------
    point_dict: dict
        A dict of the slot coordinates
    """

    Rext = self.get_Rext()

    # Compute point coordinates
    Zc0 = Rext - self.H0 + self.R1
    Z0 = Rext - self.H0
    Z1 = (self.R1 * exp(1j * (-pi + self.W0 / 2))) + Zc0
    Z2 = (self.R1 * exp(1j * (pi - self.W0 / 2))) + Zc0
    Z4 = ((self.R1 + self.H1) * exp(1j * (-pi + self.W0 / 2))) + Zc0
    Z3 = ((self.R1 + self.H1) * exp(1j * (pi - self.W0 / 2))) + Zc0

    point_dict = dict()
    point_dict["Z0"] = Z0
    point_dict["Z1"] = Z1
    point_dict["Z2"] = Z2
    point_dict["Z3"] = Z3
    point_dict["Z4"] = Z4
    point_dict["Zc0"] = Zc0

    return point_dict



# -*- coding: utf-8 -*-

import numpy as np


def is_inside(self, vertice, point, normal_t=None):
    """Check if a point is inside the cell.

    Parameters
    ----------
    self : RefSegmentP1
        a RefSegmentP1 object
    vertice : ndarray
        vertice of the cell
    point : ndarray
        coordinates of a point
    normal : ndarray
        normal of another cell. Additional facultative criterion.

        Returns
    -------
    is_inside : bool
        true if the point is inside the cell
    """

    epsilon = self.epsilon

    point_ref = self.get_ref_point(vertice, point)
    s = point_ref[0]
    t = point_ref[1]

    a = abs(s) - (1 + epsilon)
    b = abs(t) - (epsilon * ((1 - s ** 2) + 1))
    is_inside = (a < 0) & (b < 0)  # >= in case the point is "just" on the border

    # Check that normals are almost aligned
    if normal_t is not None:
        normal_s = self.get_normal(vertice)
        scal_st = np.dot(normal_t[0:2], normal_s)
        is_colinear = abs(scal_st) > 1 - 2 * epsilon
        is_inside = is_inside & is_colinear

    return is_inside, a, b



# -*- coding: utf-8 -*-

from PySide2.QtCore import Signal
from PySide2.QtWidgets import QMessageBox, QWidget

from .....GUI.Dialog.DMachineSetup.SWindParam.Gen_SWindParam import Gen_SWindParam


class SWindParam(Gen_SWindParam, QWidget):
    """Step to define the winding parameters"""

    # Signal to DMachineSetup to know that the save popup is needed
    saveNeeded = Signal()
    # Information for DMachineSetup nav
    step_name = "Winding (2)"

    def __init__(self, machine, matlib, is_stator=False):
        """Initialize the GUI according to machine

        Parameters
        ----------
        self : SWindParam
            A SWindParam widget
        machine : Machine
            current machine to edit
        matlib : MatLib
            Material Library
        is_stator : bool
            To adapt the GUI to set either the stator or the rotor
        """

        # Build the interface according to the .ui file
        QWidget.__init__(self)
        self.setupUi(self)

        # Saving arguments
        self.machine = machine
        self.matlib = matlib
        self.is_stator = is_stator

        # Fill the fields with the machine values (if they're filled)
        if self.is_stator:
            self.obj = machine.stator
        else:
            self.obj = machine.rotor

        if self.obj.winding.Ntcoil is None:
            self.si_Ntcoil.clear()
        else:
            self.si_Ntcoil.setValue(self.obj.winding.Ntcoil)

        # Adapt GUI for rotor WRSM
        if not is_stator and machine.type_machine == 9:
            self.in_Zs.hide()
            self.in_Nlay.hide()
            self.obj.winding.Npcpp = 1  # Enforce the value
            self.si_Npcpp.setEnabled(False)

        if self.obj.winding.Npcpp is None:
            self.obj.winding.Npcpp = 1  # Default value

        self.comp_output()
        self.si_Npcpp.setValue(self.obj.winding.Npcpp)

        # Connect the widget
        self.si_Ntcoil.editingFinished.connect(self.set_Ntcoil)
        self.si_Npcpp.editingFinished.connect(self.set_Npcp)

    def set_Ntcoil(self):
        """Signal to update the value of Ntcoil according to the line edit

        Parameters
        ----------
        self : SWindParam
            A SWindParam object
        """
        self.obj.winding.Ntcoil = self.si_Ntcoil.value()
        self.comp_output()
        # Notify the machine GUI that the machine has changed
        self.saveNeeded.emit()

    def set_Npcp(self):
        """Signal to update the value of Npcp according to the line edit

        Parameters
        ----------
        self : SWindParam
            A SWindParam object
        """
        self.obj.winding.Npcpp = self.si_Npcpp.value()
        self.comp_output()
        # Notify the machine GUI that the machine has changed
        self.saveNeeded.emit()

    def comp_output(self):
        """Compute and display the winding output

        Parameters
        ----------
        self : SWindParam
            A SWindParam object
        """

        Zs = self.obj.slot.Zs
        qs = self.obj.winding.qs

        try:
            Ntspc = str(self.obj.winding.comp_Ntspc(Zs))
        except:
            Ntspc = "?"
        try:
            Ncspc = str(self.obj.winding.comp_Ncspc(Zs))
        except:
            Ncspc = "?"
        try:
            (Nrad, Ntan) = self.obj.winding.get_dim_wind()
            Nlay = str(Nrad * Ntan)
        except:
            Nlay = "?"

        self.in_Zs.setText(self.tr("Zs: ") + str(Zs))
        self.in_qs.setText(self.tr("qs: ") + str(qs))
        self.out_Ncspc.setText(self.tr("Ncspc: ") + Ncspc)
        self.out_Ntspc.setText(self.tr("Ntspc: ") + Ntspc)
        self.in_Nlay.setText(self.tr("Nlay: ") + str(Nlay))

    @staticmethod
    def check(lamination):
        """Check that the current lamination have all the needed field set

        Parameters
        ----------
        lamination : Lamination
            Lamination to check

        Returns
        -------
        error: str
            Error message (return None if no error)
        """

        try:
            obj = lamination.winding  # For readibility

            # Check that everything is set
            if obj.Ntcoil is None:
                return "You must set Ntcoil !"
            if obj.Npcpp is None:
                return "You must set Npcpp !"
        except Exception as e:
            return str(e)



# -*- coding: utf-8 -*-


def has_magnet(self):
    """Return if the Hole has magnet

    Parameters
    ----------
    self : HoleMag
        A HoleMag object

    Returns
    -------
    has_magnet : bool
        True

    """

    return True



# -*- coding: utf-8 -*-


def comp_width_airgap_mec(self):
    """Compute the mechanical airgap (mag_airgap - magnet or ring)

    Parameters
    ----------
    self: Machine
        Machine object
    Returns
    -------
    mec_gap: float
        The mechanical airgap [m]

    """

    if self.rotor.is_internal:
        return self.stator.comp_radius_mec() - self.rotor.comp_radius_mec()
    else:
        return self.rotor.comp_radius_mec() - self.stator.comp_radius_mec()



# -*- coding: utf-8 -*-
from numpy import imag, pi, exp
from numpy import abs as np_abs
from ....Classes.Circle import Circle
from ....Classes.SurfLine import SurfLine
from ....Classes.Arc import Arc
from ....Classes.Arc1 import Arc1
from ....Classes.Segment import Segment


def build_geometry_half_tooth(self, is_top=False, alpha=0, delta=0):
    """Build the geometry of a Half Tooth

    Parameters
    ----------
    self : LamSlot
        a LamSlot object
    is_top : bool
        To select the part of the tooth (X>0 or X <0)
    alpha : float
        Angle for rotation [rad]
    delta : complex
        Complex value for translation

    Returns
    -------
    surf_list : list
        list of surfaces needed to draw the half tooth including slot base

    """

    # getting Number of Slot
    Zs = self.Zs
    Rbo = self.get_Rbo()

    slot_list = self.build_geometry()
    top_list = list()
    bot_list = list()

    for line in slot_list:
        Zbegin = line.get_begin()
        Zend = line.get_end()
        # append line
        if imag(Zbegin) <= 0 and imag(Zend) <= 0:
            top_list.append(line)
        elif imag(Zbegin) >= 0 and imag(Zend) >= 0:
            bot_list.append(line)
        else:  # The line cross the X axis => split
            # Copy the line (split_half modify the object)
            line2 = type(line)(init_dict=line.as_dict())
            # Split the lines
            line.split_half(is_begin=True)
            line2.split_half(is_begin=False)
            if imag(Zbegin) < 0:
                top_list.append(line)
                bot_list.append(line2)
            else:
                top_list.append(line2)
                bot_list.append(line)

    # add bore lines
    Zbo = Rbo * exp(-1j * pi / Zs)
    if np_abs(Zbo - top_list[0].get_begin()) > 1e-6:
        top_list.insert(
            0, Arc1(Zbo, top_list[0].get_begin(), Rbo, label="Tooth_bore_arc_top")
        )
    Zbo = Rbo * exp(1j * pi / Zs)
    if np_abs(Zbo - bot_list[-1].get_end()) > 1e-6:
        bot_list.append(
            Arc1(bot_list[-1].get_end(), Zbo, Rbo, label="Tooth_bore_arc_bot")
        )

    # Select the lines to return
    if is_top:
        tooth_list = top_list
        T_angle = pi / Zs
    else:
        tooth_list = bot_list
        T_angle = -pi / Zs

    # rotate
    for line in tooth_list:
        line.rotate(alpha + T_angle)

    # translate
    for line in tooth_list:
        line.translate(delta)

    return tooth_list



# -*- coding: utf-8 -*-


def comp_height_yoke(self):
    """Compute the yoke height

    Parameters
    ----------
    self : LamSlot
        A LamSlot object

    Returns
    -------
    Hy: float
        yoke height [m]

    """
    if self.slot is not None and self.slot.Zs > 0:
        return self.Rext - self.Rint - self.slot.comp_height()
    else:
        return self.Rext - self.Rint



# -*- coding: utf-8 -*-
import matplotlib.pyplot as plt
from ....Functions.init_fig import init_fig


def plot(
    self,
    fig=None,
    ax=None,
    sym=1,
    alpha=0,
    delta=0,
    is_edge_only=False,
    comp_machine=None,
    is_show_fig=True,
    save_path=None,
):
    """Plot the Machine in a matplotlib fig

    Parameters
    ----------
    self : Machine
        A Machine object
    fig : Matplotlib.figure.Figure
        existing figure to use if None create a new one
    ax : Matplotlib.axes.Axes object
        Axis on which to plot the data
    sym : int
        Symmetry factor (1= full machine, 2= half of the machine...)
    alpha : float
        Angle for rotation [rad]
    delta : complex
        Complex value for translation
    is_edge_only: bool
        To plot transparent Patches
    comp_machine : Machine
        A machine to plot in transparency on top of the self machine
    is_show_fig : bool
        To call show at the end of the method
    save_path : str
        full path including folder, name and extension of the file to save if save_path is not None
    """

    (fig, ax, _, _) = init_fig(fig=fig, ax=ax, shape="rectangle")

    # Call each plot method to properly set the legend
    if self.frame is not None:
        self.frame.plot(
            fig=fig,
            ax=ax,
            sym=sym,
            alpha=alpha,
            delta=delta,
            is_edge_only=is_edge_only,
            is_show_fig=False,
        )
        Wfra = self.frame.comp_height_eq()
    else:
        Wfra = 0

    # Determin order of plotting parts
    lam_list = self.get_lam_list(is_int_to_ext=True)

    Rext = lam_list[-1].Rext
    for lam in lam_list[::-1]:
        lam.plot(
            fig=fig,
            ax=ax,
            sym=sym,
            alpha=alpha,
            delta=delta,
            is_edge_only=is_edge_only,
            is_show_fig=False,
        )

    if lam_list[0].Rint > 0 and self.shaft is not None:
        self.shaft.plot(
            fig=fig,
            ax=ax,
            sym=sym,
            alpha=alpha,
            delta=delta,
            is_edge_only=is_edge_only,
            is_show_fig=False,
        )

    Lim = (Rext + Wfra) * 1.5  # Axes limit for plot

    if comp_machine is not None:
        comp_machine.rotor.plot(
            fig,
            ax,
            sym=sym,
            alpha=alpha,
            delta=delta,
            is_edge_only=True,
            is_show_fig=is_show_fig,
        )
        comp_machine.stator.plot(
            fig,
            ax,
            sym=sym,
            alpha=alpha,
            delta=delta,
            is_edge_only=True,
            is_show_fig=is_show_fig,
        )

    ax.set_xlabel("(m)")
    ax.set_ylabel("(m)")
    ax.set_title(self.name)

    # Axis Setup
    plt.axis("equal")

    # The Lamination is centered in the figure
    ax.set_xlim(-Lim, Lim)
    ax.set_ylim(-Lim, Lim)

    # Set Windows title
    if self.name not in ["", None]:
        fig.canvas.set_window_title(self.name + " plot machine")

    if save_path is not None:
        fig.savefig(save_path)
        plt.close()

    if is_show_fig:
        fig.show()



# -*- coding: utf-8 -*-

from matplotlib.pyplot import axis, legend

from ....Functions.init_fig import init_fig
from ....definitions import config_dict

PATCH_EDGE = config_dict["PLOT"]["COLOR_DICT"]["PATCH_EDGE"]
PATCH_COLOR = config_dict["PLOT"]["COLOR_DICT"]["PATCH_COLOR"]


def plot(
    self,
    fig=None,
    ax=None,
    color=PATCH_COLOR,
    edgecolor=PATCH_EDGE,
    is_edge_only=False,
    linestyle=None,
    is_disp_point_ref=False,
    is_show_fig=True,
):
    """Plot the Surface patch in a matplotlib fig

    Parameters
    ----------
    self : Surface
        A Surface object
    fig :
        if None, open a new fig and plot, else add to the
        current one (Default value = None)
    color :
        the color of the patch (Default value = PATCH_COLOR)
    edgecolor :
        the edge color of the patch (Default value = PATCH_EDGE)
    is_edge_only: bool
        To set the transparancy of the face color to 0 and 1 for the edge color
    linestyle : str
        Line style of the edge {'-', '--', '-.', ':', '', (offset, on-off-seq), ...}
    is_disp_point_ref : bool
        True to add the point_ref
    is_show_fig : bool
        To call show at the end of the methods

    Returns
    -------
    None
    """

    (fig, axes, patch_leg, label_leg) = init_fig(fig, ax)
    axes.set_xlabel("(m)")
    axes.set_ylabel("(m)")

    patches = self.get_patches(
        color=color, edgecolor=edgecolor, is_edge_only=is_edge_only, linestyle=linestyle
    )
    for patch in patches:
        axes.add_patch(patch)

    if is_disp_point_ref:
        axes.plot(self.point_ref.real, self.point_ref.imag, "kx")
    # Axis Setup
    axes.axis("equal")

    if is_show_fig:
        fig.show()



# -*- coding: utf-8 -*-


def __setitem__(self, key, value):
    """Called to implement assignment to self[key]."""
    if isinstance(value, tuple):
        self._statements.__setitem__(key, value[0])
        if len(tuple) >= 1:
            self._comments.__setitem__(key, value[1])
        else:
            self._comments.__setitem__(key, None)
    else:
        self._statements.__setitem__(key, value)
        self._comments.__setitem__(key, None)
